% Change policy regime in line 550
% 1 = MC+OF
% 2 = LC+OF
% 3 = MC+PF
% 4 = LC+PF
% 5 = MC+AF
% 6 = LC+AF


disp('---------------------------------------------------------------')
disp(' ');
clear all;
clc;

xxx=1000; %covergence starting value
Cdiff    = 1;
Fweight  = 1;
Vweight  = 1; 
ConvCrit = 1e-12|1e-12|1e-12;
CritLags = 0;
%--------------------------------------------------------------------
% Parameters
%---------------------------------------------------------------------
rho_Tr	    =	0.968	;
rho_tech	=	0.274	;
rho_cost	=	0.938	;
rho_perf	=	0.938	;
rho_gov	    =	0.986	;

habit	=	0.802	;
sigma	=	3.102	;
phi	    =	2.000	;
calvo	=	0.780	;
index	=	0.353	;

% passive fiscal rule  
debtp	=	0.0447	;
rho_tau	=	0.9642  ;

% active fiscal rule 
debtp0	    =	0.000	;
rho_tau0	=	0.932	;
taug_chi	=	0.001	;
yg_chi      =   0.000   ; % not used 

% Government spending rule 
y_chi	    =	0.0	; % not used 

% shock s.d. 
s_Tr	    = [0.303  0.303];
s_tech	    = [0.684  1.218];
s_cost	    = [0.545  1.660];
s_perf	    = [0.804  2.318];
s_gov	    = [0.161  0.161];
s_taustar   = [0.234  0.234];
s_debt      = [2.558  2.558];

% this part is common to both fiscal and monetary policies. 
lambd1	=	0.221	;
lambd3	=	0.256	;
lambd5  =   0.422   ;

% monetary policy weights on bad and good regimes
lambd4     =    1.000; % good monetary regime
lambd44    =    0.611; % bad monetary regime 
lambd6     =    0.739; % interest rate smoothing term
lambd44F   =	0.298; % inflation aversion of fiscal policy maker
lambd66F   =	0.699; % tax rate smoothing term  

% fiscal policy weights six regimes
lambd1F	   =   [lambd1 lambd1 0 0 0 0]; % output 1
lambd3F    =   [lambd3 lambd3 0 0 0 0]; % output 2
lambd4F    =   [lambd44F lambd44F   0 0 0 0]; % inflation stabilisation term 1  
lambd5F    =   [lambd5 lambd5 0 0 0 0];   % inflation stabilisation term 2 
lambd6F    =   [0 0 0 0 0 0];           % intrest rate smoothing term      
lambd7F	   =   [lambd66F lambd66F 0 0 0 0]; % tax rate smoothing term 
lambd8F    =   [0 0           1 1 0 0]; % passive fiscal
lambd9F    =   [0 0           0 0 1 1]; % active fiscal

% monetary policy weights six regimes
lambd1M	   =	[lambd1 lambd1 lambd1 lambd1 lambd1 lambd1]; % output 1
lambd3M    =	[lambd3 lambd3 lambd3 lambd3 lambd3 lambd3]; % output 2
lambd4M    =    [lambd4 lambd44 lambd4 lambd44 lambd4 lambd44]; % inflation stabilisation term 1   
lambd5M    =    [lambd5 lambd5 lambd5 lambd5 lambd5 lambd5]; % inflation stabilisation term 2
lambd6M    =	[lambd6 lambd6 lambd6 lambd6 lambd6 lambd6]; % intrest rate smoothing term  
lambd7M    =    [0 0 0 0 0 0];                             % tax rate smoothing term

% fiscal policy transition probabilities
z11	=	0.875	;
z12	=	0.004	;
z22	=	0.966	;
z21	=	0.007	;
z33	=	0.916	;
z32	=	0.001	;

% monetary policy transition probabilities
q11	=	0.962	;
q22	=	0.956	;

% Markov Transition Matrix
strans = [z11           (1-z21-z22)      z32
          z12            z22            (1-z33-z32)
         (1-z11-z12)     z21             z33];
				 		
strans2 = [q11  (1-q22)
          (1-q11) q22];
      
PP = kron(strans,strans2); 
Ps = PP'^10000;
ppp = Ps(1,:);   

% Data means      
rr     = 2;
p_i_ss = 2;
ga_mma = 0.46;
g_y    = 0.08;  
pmb_y  = 1.24;
pmb_ya = 1.24/4; 
tau    = 0.175;
tr     = 0.0919;

p11	=	0.965;
p22	=	0.894;

strans3 = [p11  (1-p22)
          (1-p11) p22];
      
PPs = strans3'^10000;      
pppp = PPs(1,:);  
%********************************************************************
%additional parameters 
%********************************************************************/ 
bet   = 1/(1+rr/400); 
beta  = 1/(1+rr/400); 
pibar = p_i_ss/400+1;
gam   = ga_mma/100+1;
rho   = (1-0.05)/beta;
Dinom = calvo*(1 + beta*index) + (1-calvo)*index; 
xf    = calvo/Dinom;
xb    = index/Dinom;
kc    = ((1-calvo)*(1-index)*(1-calvo*beta))/Dinom;

%---------------------------------------------------------------------
ns = 6;          %number of modes
nX = 17;         %predeterminate
nxx = 8;         %jump
n = nX + nxx;    %total number of predeterminate and jump varaibles
p = 2;           %total number of instruments
nld = 1;         %number of leader
nfl = 1;         %number of follower
target  = 7;     %number of targeted variables of monetary(follower) 
target2 = 9;     %number of targeted variables of fiscal(leader)
ny      = 7;     %number of variables
%---------------------------------------------------------------------
% variable indices
vi_Tr	=	1	;
vi_tech	=	2	;
vi_cost	=	3	;
vi_perf	=	4	;
vi_gsh	=	5	;
vi_taustarsh	=	6	;
vi_debtsh	=	7	;
vi_glag	=	8	;
vi_taulag	=	9	;
vi_debtlag	=	10	;
vi_ylag	=	11	;
vi_ilag	=	12	;
vi_clag	=	13	;
vi_pilag	=	14	;
vi_pmlag	=	15	;
vi_taulagstart	=	16	;
vi_taulagstart2	=	17	;
vi_taustart	=	18	;
vi_taustart2	=	19	;
vi_g	=	20	;
vi_debt	=	21	;
vi_y	=	22	;
vi_pm	=	23	;
vi_c	=	24	;
vi_pi	=	25	;
vi_tau	=	26	;
vi_i	=	27	;
%---------------------------------------------------------------------
% instrument 
in_tau  = 1;  
in_i    = 2;  
%---------------------------------------------------------------------
% Equation indices
eq_Tr	=	1	;
eq_tech	=	2	;
eq_cost	=	3	;
eq_perf	=	4	;
eq_gsh	=	5	;
eq_taustarsh	=	6	;
eq_debtsh	=	7	;
eq_glag	=	8	;
eq_taulag	=	9	;
eq_debtlag	=	10	;
eq_ylag	=	11	;
eq_ilag	=	12	;
eq_clag	=	13	;
eq_pilag	=	14	;
eq_pmlag	=	15	;
eq_taulagstart	=	16	;
eq_taulagstart2	=	17	;
eq_taustart	=	18	;
eq_taustart2	=	19	;
eq_g	=	20	;
eq_debt	=	21	;
eq_y	=	22	;
eq_pm	=	23	;
eq_c	=	24	;
eq_pi	=	25	;
%---------------------------------------------------------------------
%initialize matrices 
Am = zeros(n,n,ns);
Bm = zeros(n,n,ns);
Cm = zeros(n,p,ns);
HH = zeros(nxx,nxx,ns);
lossld = zeros(target2,target2,ns); % fiscal 
lossfl = zeros(target,target,ns); % monetary

% two shocks regimes 
nss = 2; 
C  = zeros(nX,nX,nss);

for ii = 1:nss 
%%%**********************************************************
%%%  4 shocks 
%%%**********************************************************
C(vi_Tr,vi_Tr,ii)                 = s_Tr(ii);
C(vi_tech,vi_tech,ii)             = s_tech(ii);
C(vi_cost,vi_cost,ii)             = s_cost(ii); 
C(vi_perf,vi_perf,ii)             = s_perf(ii); 
C(vi_gsh,vi_gsh,ii)               = s_gov(ii); 
C(vi_taustarsh,vi_taustarsh,ii)   = s_taustar(ii);
C(vi_debtsh,vi_debtsh,ii)         = s_debt(ii); 
end

CC = pppp(1,1)*C(:,:,1) + pppp(1,2)*C(:,:,2);
%CC = C(:,:,1);   % low volatility
%CC = C(:,:,2);   % high volatility

% The OF and PF regimes
for i = 1:4  
% transfers 
Am(eq_Tr,vi_Tr,i) = 1;                  Bm(eq_Tr,vi_Tr,i) = rho_Tr;
% cost push 
Am(eq_cost,vi_cost,i) = 1;              Bm(eq_cost,vi_cost,i) = rho_cost;
% perference shocks
Am(eq_perf,vi_perf,i) = 1;              Bm(eq_perf,vi_perf,i) = rho_perf;
% technology
Am(eq_tech,vi_tech,i) = 1;              Bm(eq_tech,vi_tech,i) = rho_tech;
% government shocks
Am(eq_gsh,vi_gsh,i) = 1;                Bm(eq_gsh,vi_gsh,i) = 0.0;
% tau shocks
Am(eq_taustarsh,vi_taustarsh,i) = 1;    Bm(eq_taustarsh,vi_taustarsh,i) = 0.0;
% debt shocks
Am(eq_debtsh,vi_debtsh,i) = 1;          Bm(eq_debtsh,vi_debtsh,i) = 0.0;

%  passive tax rule 
Bm(eq_taustart,vi_taustart,i)     = -1;
Bm(eq_taustart,vi_taulagstart,i)  = rho_tau;
Bm(eq_taustart,vi_debtlag,i)      = (1-rho_tau)*debtp; 
Bm(eq_taustart,vi_y,i)            = (1-rho_tau)*taug_chi; 
Bm(eq_taustart,vi_g,i)            = (1-rho_tau)*yg_chi; 
Bm(eq_taustart,vi_taustarsh,i)    = 1; 

%  active tax rule 
Bm(eq_taustart2,vi_taustart2,i)     = -1;
Bm(eq_taustart2,vi_taulagstart2,i)  = rho_tau0;
Bm(eq_taustart2,vi_debtlag,i)       = (1-rho_tau0)*debtp0; 
Bm(eq_taustart2,vi_y,i)             = (1-rho_tau0)*taug_chi; 
Bm(eq_taustart2,vi_g,i)             = (1-rho_tau0)*yg_chi; 
Bm(eq_taustart2,vi_taustarsh,i)     = 1; 

% government spending rule 
Bm(eq_g,vi_g,i)       = -1; 
Bm(eq_g,vi_glag,i)    = rho_gov;
Bm(eq_g,vi_debtlag,i) = (1-rho_gov)*y_chi; 
Bm(eq_g,vi_gsh,i)     = 1;

% budget contraint
Bm(eq_debt,vi_debt,i)    =  -1;
Bm(eq_debt,vi_debtlag,i) =  1/beta;
Bm(eq_debt,vi_ylag,i)    =  pmb_y/beta;
Bm(eq_debt,vi_y,i)       = -pmb_y/beta; 
Bm(eq_debt,vi_pi,i)      = -pmb_y/beta;
Bm(eq_debt,vi_tech,i)    = -pmb_y/beta;

Bm(eq_debt,vi_Tr,i)      =  1;
Cm(eq_debt,in_tau,i)     = -1;
Bm(eq_debt,vi_g,i)       =  1;

Bm(eq_debt,vi_pm,i)      =  (pmb_y*rho)/(gam*pibar); 
Bm(eq_debt,vi_pmlag,i)   =  -pmb_y/beta;  

Bm(eq_debt,vi_debtsh,i) = 1;

%  output
Bm(eq_y,vi_y,i) = -1;
Bm(eq_y,vi_c,i) =  1;
Bm(eq_y,vi_g,i) =  1/(1-g_y);                

% linking equation = the number of jump variables
Am(eq_glag,vi_glag,i)        = 1;            Bm(eq_glag,vi_g,i)        = 1;
Am(eq_debtlag,vi_debtlag,i)  = 1;            Bm(eq_debtlag,vi_debt,i)  = 1;
Am(eq_ylag,vi_ylag,i)        = 1;            Bm(eq_ylag,vi_y,i)        = 1;
Am(eq_pmlag,vi_pmlag,i)      = 1;            Bm(eq_pmlag,vi_pm,i)      = 1;
Am(eq_pilag,vi_pilag,i)      = 1;            Bm(eq_pilag,vi_pi,i)      = 1;
Am(eq_clag,vi_clag,i)        = 1;            Bm(eq_clag,vi_c,i)        = 1;
Am(eq_taulagstart,vi_taulagstart,i)   = 1;   Bm(eq_taulagstart,vi_taustart,i)    = 1;
Am(eq_taulagstart2,vi_taulagstart2,i) = 1;   Bm(eq_taulagstart2,vi_taustart2,i)  = 1;

% instruments
Am(eq_taulag,vi_taulag,i) = 1;      Cm(eq_taulag,in_tau,i) = 1;
Am(eq_ilag,vi_ilag,i)    = 1;       Cm(eq_ilag,in_i,i)    = 1;

%  Consumption Euler Equation
Am(eq_c,vi_c,i)    =  -1;
Am(eq_c,vi_pi,i)   =  -(1-habit)/sigma; 
Bm(eq_c,vi_tech,i) =   rho_tech*(1-habit)/sigma;
Bm(eq_c,vi_c,i)    =  -(1+habit);
Bm(eq_c,vi_clag,i) =   habit;
Bm(eq_c,vi_perf,i) =  -(1-habit)*(1-rho_perf);
Cm(eq_c,in_i,i)    =  -(1-habit)/sigma;

% PC
Am(eq_pi,vi_pi,i)     = -xf*beta; 
Bm(eq_pi,vi_pi,i)     = -1;
Bm(eq_pi,vi_pilag,i)  =  xb;
Bm(eq_pi,vi_c,i)      = (kc*sigma)/(1-habit); 
Bm(eq_pi,vi_clag,i)   = -kc*sigma*habit/(1-habit); 
Bm(eq_pi,vi_y,i)      =  kc*phi;
Bm(eq_pi,vi_cost,i)   =  kc;
Cm(eq_pi,in_tau,i)    =  kc/(1-tau); 

% PM
Am(eq_pm,vi_pm,i) = rho*beta/(pibar*gam);  
Bm(eq_pm,vi_pm,i) = 1;
Cm(eq_pm,in_i,i)  = 1;

HH(:,:,i) = Am(nX+1:nX+nxx,nX+1:nX+nxx,i);
end


% The AF regime - I left these two regimes so you can change budget
% constraint. 

for i = 5:6
% transfers 
Am(eq_Tr,vi_Tr,i) = 1;                  Bm(eq_Tr,vi_Tr,i) = rho_Tr;
% cost push 
Am(eq_cost,vi_cost,i) = 1;              Bm(eq_cost,vi_cost,i) = rho_cost;
% perference shocks
Am(eq_perf,vi_perf,i) = 1;              Bm(eq_perf,vi_perf,i) = rho_perf;
% technology
Am(eq_tech,vi_tech,i) = 1;              Bm(eq_tech,vi_tech,i) = rho_tech;
% government shocks
Am(eq_gsh,vi_gsh,i) = 1;                Bm(eq_gsh,vi_gsh,i) = 0.0;
% tau shocks
Am(eq_taustarsh,vi_taustarsh,i) = 1;    Bm(eq_taustarsh,vi_taustarsh,i) = 0.0;
% debt shocks
Am(eq_debtsh,vi_debtsh,i) = 1;          Bm(eq_debtsh,vi_debtsh,i) = 0.0;

%  passive tax rule 
Bm(eq_taustart,vi_taustart,i)     = -1;
Bm(eq_taustart,vi_taulagstart,i)  = rho_tau;
Bm(eq_taustart,vi_debtlag,i)      = (1-rho_tau)*debtp; 
Bm(eq_taustart,vi_y,i)            = (1-rho_tau)*taug_chi; 
Bm(eq_taustart,vi_g,i)            = (1-rho_tau)*yg_chi; 
Bm(eq_taustart,vi_taustarsh,i)    = 1; 

%  active tax rule 
Bm(eq_taustart2,vi_taustart2,i)     = -1;
Bm(eq_taustart2,vi_taulagstart2,i)  = rho_tau0;
Bm(eq_taustart2,vi_debtlag,i)       = (1-rho_tau0)*debtp0; 
Bm(eq_taustart2,vi_y,i)             = (1-rho_tau0)*taug_chi; 
Bm(eq_taustart2,vi_g,i)             = (1-rho_tau0)*yg_chi; 
Bm(eq_taustart2,vi_taustarsh,i)     = 1; 

% government spending rule 
Bm(eq_g,vi_g,i)       = -1; 
Bm(eq_g,vi_glag,i)    = rho_gov;
Bm(eq_g,vi_debtlag,i) = (1-rho_gov)*y_chi; 
Bm(eq_g,vi_gsh,i)     = 1;

% budget contraint
Bm(eq_debt,vi_debt,i)    =  -1;
Bm(eq_debt,vi_debtlag,i) =  1/beta;
Bm(eq_debt,vi_ylag,i)    =  pmb_y/beta;
Bm(eq_debt,vi_y,i)       = -pmb_y/beta; 
Bm(eq_debt,vi_pi,i)      = -pmb_y/beta;
Bm(eq_debt,vi_tech,i)    = -pmb_y/beta;

Bm(eq_debt,vi_Tr,i)       =  1;
%Cm(eq_debt,in_tau,i)     = -1; %switch back to instrument, comment out the line below.  
Bm(eq_debt,vi_taustart2,i) = -1;
Bm(eq_debt,vi_g,i)       =  1;

Bm(eq_debt,vi_pm,i)      =  (pmb_y*rho)/(gam*pibar); 
Bm(eq_debt,vi_pmlag,i)   =  -pmb_y/beta;  

Bm(eq_debt,vi_debtsh,i) = 1;

%  output
Bm(eq_y,vi_y,i) = -1;
Bm(eq_y,vi_c,i) =  1;
Bm(eq_y,vi_g,i) =  1/(1-g_y);                

% linking equation = the number of jump variables
Am(eq_glag,vi_glag,i)        = 1;            Bm(eq_glag,vi_g,i)        = 1;
Am(eq_debtlag,vi_debtlag,i)  = 1;            Bm(eq_debtlag,vi_debt,i)  = 1;
Am(eq_ylag,vi_ylag,i)        = 1;            Bm(eq_ylag,vi_y,i)        = 1;
Am(eq_pmlag,vi_pmlag,i)      = 1;            Bm(eq_pmlag,vi_pm,i)      = 1;
Am(eq_pilag,vi_pilag,i)      = 1;            Bm(eq_pilag,vi_pi,i)      = 1;
Am(eq_clag,vi_clag,i)        = 1;            Bm(eq_clag,vi_c,i)        = 1;
Am(eq_taulagstart,vi_taulagstart,i)   = 1;   Bm(eq_taulagstart,vi_taustart,i)    = 1;
Am(eq_taulagstart2,vi_taulagstart2,i) = 1;   Bm(eq_taulagstart2,vi_taustart2,i)  = 1;

% instruments
Am(eq_taulag,vi_taulag,i) = 1;      Cm(eq_taulag,in_tau,i) = 1;
Am(eq_ilag,vi_ilag,i)    = 1;       Cm(eq_ilag,in_i,i)    = 1;

%  Consumption Euler Equation
Am(eq_c,vi_c,i)    =  -1;
Am(eq_c,vi_pi,i)   =  -(1-habit)/sigma; 
Bm(eq_c,vi_tech,i) =   rho_tech*(1-habit)/sigma;
Bm(eq_c,vi_c,i)    =  -(1+habit);
Bm(eq_c,vi_clag,i) =   habit;
Bm(eq_c,vi_perf,i) =  -(1-habit)*(1-rho_perf);
Cm(eq_c,in_i,i)    =  -(1-habit)/sigma;

% PC
Am(eq_pi,vi_pi,i)     = -xf*beta; 
Bm(eq_pi,vi_pi,i)     = -1;
Bm(eq_pi,vi_pilag,i)  =  xb;
Bm(eq_pi,vi_c,i)      = (kc*sigma)/(1-habit); 
Bm(eq_pi,vi_clag,i)   = -kc*sigma*habit/(1-habit); 
Bm(eq_pi,vi_y,i)      =  kc*phi;
Bm(eq_pi,vi_cost,i)   =  kc;
Cm(eq_pi,in_tau,i)    =  kc/(1-tau); 

% PM
Am(eq_pm,vi_pm,i) = rho*beta/(pibar*gam);  
Bm(eq_pm,vi_pm,i) = 1;
Cm(eq_pm,in_i,i)  = 1;

HH(:,:,i) = Am(nX+1:nX+nxx,nX+1:nX+nxx,i);
end


% We use the estimated weights  
df = zeros(target2,n+p,ns); % Fiscal policy 
d  = zeros(target,n+p,ns);  % Money policy

for i = 1:ns
    
   lossld(:,:,i) = diag([lambd1F(i) 0 lambd3F(i) lambd4F(i) lambd5F(i) 0 lambd7F(i)	lambd8F(i)  lambd9F(i)]); % fiscal policy
   
   df(1,vi_perf,i) = 1;    
   df(1,vi_c,i)   = 1/(1-habit); 
   df(1,vi_clag,i) = -habit/(1-habit);

   df(2,vi_perf,i) = 1; 
   df(2,vi_g,i)    = 1; 

   df(3,vi_y,i)   = 1;
   df(3,vi_perf,i) = -sigma/phi;

   df(4,vi_pi,i) = 1;

   df(5,vi_pi,i)    = 1;
   df(5,vi_pilag,i) = -1;

   df(6,vi_ilag,i) = -1;
   df(6,vi_i,i)    = 1;

   df(7,vi_tau,i)    =  1;
   df(7,vi_taulag,i) = -1;

   df(8,vi_tau,i)     =  1;
   df(8,vi_taustart,i) = -1;  %passive fiscal rule 

   df(9,vi_tau,i)      =  1;
   df(9,vi_taustart2,i) = -1; %active fiscal rule 
   
   lossfl(:,:,i) = diag([lambd1M(i) 0 lambd3M(i) lambd4M(i) lambd5M(i) lambd6M(i) 0 ]); % monetary policy
   
  d(1,vi_perf,i)   = 1; 
  d(1,vi_c,i)      = 1/(1-habit); 
  d(1,vi_clag,i)   = -habit/(1-habit);

  d(2,vi_perf,i)   = 1; 
  d(2,vi_g,i)      = 1; 

  d(3,vi_y,i)      = 1;
  d(3,vi_perf,i)   = -sigma/phi;

  d(4,vi_pi,i)     = 1;

  d(5,vi_pi,i)     =  1;
  d(5,vi_pilag,i)  = -1;

  d(6,vi_i,i)      =  1;
  d(6,vi_ilag,i)   = -1;

  d(7,vi_tau,i)    =  1;
  d(7,vi_taulag,i) = -1;
end

% welfare functions is here
for j = 1:ns   
   qhld(:,:,j) = df(:,:,j)'*lossld(:,:,j)*df(:,:,j); 
   qhfl(:,:,j) = d(:,:,j)' *lossfl(:,:,j)*d(:,:,j);
end

Qld  = qhld(1:nX+nxx,1:nX+nxx,:);  
Pld  = qhld(1:nX+nxx,nX+nxx+1:nX+nxx+p,:);
Rld  = qhld(nX+nxx+1:nX+nxx+p,nX+nxx+1:nX+nxx+p,:);

Qfl = qhfl(1:nX+nxx,1:nX+nxx,:); 
Pfl = qhfl(1:nX+nxx,nX+nxx+1:nX+nxx+p,:);
Rfl = qhfl(nX+nxx+1:nX+nxx+p,nX+nxx+1:nX+nxx+p,:);

Vflt1 = ones(nX,nX)*1;  
Vldt1 = ones(nX,nX)*1;  
Ct1 = ones(nxx,nX)*1;   

%% the following are the leadership code with transition probabilities

% initialise values
indexx = 0;  indicator = 0;
n = nX + nxx; 
k = nld + nfl; 

Fld = ones(nld,nX)*10;  %leader's X
Ffl = ones(nfl,nX)*10;  %follower's X
Lfl = ones(nfl,nld)*10; %follower's uL

%% select Regimes 
% 1 = MC+OF
% 2 = LC+OF
% 3 = MC+PF
% 4 = LC+PF
% 5 = MC+AF
% 6 = LC+AF


i = 1; 

Qld11 = Qld(1:nX,1:nX,i);    Qld12 = Qld(1:nX,nX+1:n,i);    Qld21 = Qld(nX+1:n,1:nX,i);    Qld22 = Qld(nX+1:n,nX+1:n,i); % stage_1
Rld11 = Rld(1:nld,1:nld,i);  Rld12 = Rld(1:nld,nld+1:k,i);  Rld21 = Rld(nld+1:k,1:nld,i);  Rld22 = Rld(nld+1:k,nld+1:k,i);
Pld11 = Pld(1:nX,1:nld,i);   Pld12 = Pld(1:nX,nld+1:k,i);   Pld21 = Pld(nX+1:n,1:nld,i);   Pld22 = Pld(nX+1:n,nld+1:k,i);
Qfl11 = Qfl(1:nX,1:nX,i);    Qfl12 = Qfl(1:nX,nX+1:n,i);    Qfl21 = Qfl(nX+1:n,1:nX,i);    Qfl22 = Qfl(nX+1:n,nX+1:n,i); 
Rfl11 = Rfl(1:nld,1:nld,i);  Rfl12 = Rfl(1:nld,nld+1:k,i);  Rfl21 = Rfl(nld+1:k,1:nld,i);  Rfl22 = Rfl(nld+1:k,nld+1:k,i);
Pfl11 = Pfl(1:nX,1:nld,i);   Pfl12 = Pfl(1:nX,nld+1:k,i);   Pfl21 = Pfl(nX+1:n,1:nld,i);   Pfl22 = Pfl(nX+1:n,nld+1:k,i);

H = HH(:,:,i);  
A11 = Bm(1:nX,1:nX,i);     
A12 = Bm(1:nX,nX+1:n,i);   
A21 = Bm(nX+1:n,1:nX,i);   
A22 = Bm(nX+1:n,nX+1:n,i); 
B11 = Cm(1:nX,1:nld,i);          
B12 = Cm(1:nX,(nld+1):k,i);     
B21 = Cm((nX+1):n,1:nld,i);      
B22 = Cm((nX+1):n,(nld+1):k,i);  

iter = 1; tol=1e-12; maxiter=100000; step = 0.7;
while (xxx>tol)&&(iter<maxiter) 
       
d1mat = inv(A22+H*Ct1*A12);
J   = d1mat * (H*Ct1*A11+A21); 
Kld = d1mat * (H*Ct1*B11+B21);  
Kfl = d1mat * (H*Ct1*B12+B22);  
  
Aj = A11 - A12*J;   
Bld = B11 - A12*Kld;
Bfl = B12 - A12*Kfl; 

Qfls = Qfl11 - Qfl12*J - J'*Qfl21 + J'*Qfl22*J;  
Ufls = J'*Qfl22*Kfl - Qfl12*Kfl + Pfl12-J'*Pfl22; 
Ulds = J'*Qfl22*Kld - Qfl12*Kld + Pfl11-J'*Pfl21; 
Rfls = Rfl22 + Kfl'*Qfl22*Kfl - Pfl22'*Kfl - Kfl'*Pfl22;
Pfls = Kld'*Qfl22*Kfl - Kld'*Pfl22 + Rfl12 - Pfl21'*Kfl; 
Tfls = Kld'*Qfl22*Kld - Kld'*Pfl21 - Pfl21'*Kld + Rfl11; 

Vfl = Qfls - (Fld)'*Ulds'- Ulds*(Fld)  + (Fld)'*Tfls*(Fld)+bet*(Aj'-(Fld)'*Bld')*Vflt1*(Aj-Bld*(Fld))...
      -(Ufls'-Pfls'*Fld+bet*Bfl'*Vflt1*(Aj-Bld*Fld))'*inv(Rfls + bet*Bfl'*Vflt1*Bfl)*...
      (Ufls'-Pfls'*Fld+bet*Bfl'*Vflt1*(Aj-Bld*Fld)); 

Ffln = inv(Rfls + bet*Bfl'*Vfl*Bfl) * (Ufls' + bet*Bfl'*Vfl*Aj);  
Lfln = inv(Rfls + bet*Bfl'*Vfl*Bfl) * (Pfls' + bet*Bfl'*Vfl*Bld);

Ct1n = J - Kfl*Ffln + Kfl*Lfln*Fld - Kld*Fld;  

Vflt1 = step*Vfl + (1-step)*Vflt1;
Ffl = step*Ffln + (1-step)*Ffl;
Lfl = step*Lfln + (1-step)*Lfl;
Ct1 = step*Ct1n + (1-step)*Ct1;

%% *********************************** leader ************************************/
d1mat = inv(A22+H*Ct1*A12);
J = d1mat * (H*Ct1*A11+A21); 
Kld = d1mat * (H*Ct1*B11+B21);  
Kfl = d1mat * (H*Ct1*B12+B22); 

Jx = J-Kfl*Ffl; 
Kx = Kld-Kfl*Lfl; 
Ax = A11 - B12*Ffl - A12*Jx;
Bx = B11 - B12*Lfl - A12*Kx;

Qx =  Qld11-Pld12*Ffl-Ffl'*(Pld12'-Rld22*Ffl)-(Qld12-Ffl'*Pld22')*Jx- Jx'*(Qld21-Pld22*Ffl)+ Jx'*Qld22*Jx;

Ux =  Jx'*Qld22*Kx - (Qld12 - Ffl'*Pld22')*Kx+ Pld11 - Pld12*Lfl- Ffl'*(Rld21 - Rld22*Lfl)- Jx'*Pld21 + Jx'*Pld22*Lfl;
      
Rx =  Kx'*Qld22*Kx...
      + Rld11 - Rld12*Lfl - Lfl'*(Rld21 - Rld22*Lfl)...
      - Kx'*(Pld21 - Pld22*Lfl)...
      -(Pld21'-Lfl'*Pld22')*Kx;

Vld = Qx +bet*Ax'*Vldt1*Ax-(bet*Bx'*Vldt1*Ax+Ux')'*inv(Rx + bet*Bx'*Vldt1*Bx) * (Ux' + bet*Bx'*Vldt1*Ax);
Fldn = inv(Rx + bet*Bx'*Vld*Bx) * (Ux' + bet*Bx'*Vld*Ax); 
Ct2n = J - Kfl*Ffl + Kfl*Lfl*Fldn - Kld*Fldn; 

Vadiff1 = norm(Vld-Vldt1);   %leader objective   
Vadifff1 = norm(Vfl-Vflt1);  %Leader policy 
Fadiff1 = norm(Fld-Fldn);   %Leader policy 
Cndiff1 = norm(Ct2n-Ct1);   
 
xxx =(Vadiff1+Vadifff1+Fadiff1+ Cndiff1)/4;

Fld = step*Fldn + (1-step)*Fld;
Ct1 = step*Ct2n + (1-step)*Ct1;
Vldt1 = step*Vld + (1-step)*Vldt1;
Vflt1 = step*Vfl + (1-step)*Vflt1;
    
  iter=iter+1;
end

xxx;
iter;

Cj = Ct1; 
Fa = Fld;  
Fb = Ffl; 
Lf = Lfl; 


%% ----------------------------------------------------------------
% Wjtbar
%%----------------------------------------------------------------
OMtemp = A11 - A12*Cj - B11*Fa - B12*(Fb-Lf*Fa);

OMex   =    [OMtemp              zeros(nX,nxx)  zeros(nX,nld)   zeros(nX,nfl);
            -(Cj*OMtemp)         zeros(nxx,nxx) zeros(nxx,nld)  zeros(nxx,nfl);
            -(Fa*OMtemp)         zeros(nld,nxx) zeros(nld,nld)  zeros(nld,nfl); 
            (Lf*Fa-Fb)*OMtemp    zeros(nfl,nxx) zeros(nfl,nld)  zeros(nfl,nfl)]; 
                
ONtemp  =  [-Cj;-Fa;(Lf*Fa-Fb)];        
           
OCex  =     [CC^2; -Cj*CC^2; -Fa*CC^2; (Lf*Fa-Fb)*CC^2];
OC    =     [eye(nX); -Cj; -Fa; (Lf*Fa-Fb)];

Im     =    [eye(nX)              zeros(nX,nxx)  zeros(nX,nld)   zeros(nX,nfl);
            -(Cj*eye(nX))         zeros(nxx,nxx) zeros(nxx,nld)  zeros(nxx,nfl);
            -(Fa*eye(nX))         zeros(nld,nxx) zeros(nld,nld)  zeros(nld,nfl); 
            (Lf*Fa-Fb)*eye(nX)    zeros(nfl,nxx) zeros(nfl,nld)  zeros(nfl,nfl)]; 
        

%% IRF

irfh = 101;
yyirf = zeros(irfh,ny);  
Xt = zeros(size(OMex,1),1); 
Xt(vi_Tr,1) = 0.9;
Xt=Im*Xt;

% observation indices 
eq_ygr = 1;
eq_pi  = 2;
eq_r   = 3;
eq_tau = 4;
eq_g_y = 5;
eq_b_y = 6;
eq_tr_y = 7;

%create system matrices for state space model
ZZ= zeros(ny,size(OMex,1));
ZZ(eq_ygr,vi_y)        =  1;
ZZ(eq_ygr,vi_ylag)     = -1;
ZZ(eq_ygr,vi_tech)     =  1;
ZZ(eq_pi,vi_pi)        =  4;
ZZ(eq_r,vi_i)          =  4;
ZZ(eq_tau,vi_tau)      =  1;
ZZ(eq_g_y,vi_g)        =  1;
ZZ(eq_b_y,vi_debt)     =  1/4;
ZZ(eq_tr_y,vi_Tr)      =  1;

%steady-state
DDD = zeros(ny,1);           
DDD(eq_ygr,1)  = ga_mma;
DDD(eq_pi,1)   = p_i_ss;
DDD(eq_r,1)    = p_i_ss + rr + 4*ga_mma;
DDD(eq_tau,1)  = tau*100;
DDD(eq_g_y,1)  = g_y*100;
DDD(eq_b_y,1)  = pmb_ya*100;
DDD(eq_tr_y,1) = tr*100;

yyirf(1,:) = (ZZ*Xt)'+ DDD';  

for m = 2:irfh
    Xt = OMex*Xt; 
    yyirf(m,:) = (ZZ*Xt)'+ DDD';  
end

round(yyirf, 3) % IRF 
