
% 1 = MC+OF
% 2 = LC+OF
% 3 = MC+PF
% 4 = LC+PF
% 5 = MC+AF
% 6 = LC+AF

disp('---------------------------------------------------------------')
disp(' ');
clear all;
clc;

T=10000;  %number of draws.
xxx=1000; %covergence starting value
Cdiff    = 1;
Fweight  = 1;
Vweight  = 1; 
ConvCrit = 1e-12|1e-12|1e-12;
CritLags = 0;

%--------------------------------------------------------------------
% Parameters 
%---------------------------------------------------------------------
rho_Tr	    =	0.968	;
rho_tech	=	0.274	;
rho_cost	=	0.938	;
rho_perf	=	0.938	;
rho_gov	    =	0.986	;

habit	=	0.802	;
sigma	=	3.102	;
phi	    =	2.000	;
calvo	=	0.780	;
index	=	0.353	;

% passive fiscal rule  
debtp	=	0.0447	;
rho_tau	=	0.9642  ;

% active fiscal rule 
debtp0	    =	0.000	;
rho_tau0	=	0.932	;
taug_chi	=	0.001   ; % not used - 0.001
yg_chi      =   0.000   ; % not used 

% Government spending rule
y_chi	    =	0.00	; % not used 

% shock s.d. 
s_Tr	    = [0.303  0.303];
s_tech	    = [0.684  1.218];
s_cost	    = [0.545  1.660];
s_perf	    = [0.804  2.318];
s_gov	    = [0.161  0.161];
s_taustar   = [0.234  0.234];
s_debt      = [2.558  2.558];

% this part is common to both fiscal and monetary policies. 
lambd1	=	0.221	;
lambd3	=	0.256	;
lambd5  =   0.422   ;

% monetary policy weights on bad and good regimes
lambd4     =    1; %10;     % good monetary regime
lambd44    =    0.611; % bad monetary regime 
lambd6     =    0.739; % interest rate smoothing term
lambd44F   =	0.298; % inflation aversion of fiscal policy maker
lambd66F   =	0.699; % tax rate smoothing term  

% fiscal policy weights six regimes
lambd1F	   =   [lambd1 lambd1       0 0 0 0]; % output 1
lambd3F    =   [lambd3 lambd3       0 0 0 0]; % output 2
lambd4F    =   [lambd44F lambd44F   0 0 0 0]; % inflation stabilisation term 1  
lambd5F    =   [lambd5 lambd5       0 0 0 0]; % inflation stabilisation term 2 
lambd6F    =   [0 0                 0 0 0 0]; % intrest rate smoothing term      
lambd7F	   =   [lambd66F lambd66F   0 0 0 0]; % tax rate smoothing term 
lambd8F    =   [0 0                 1 1 0 0 ]; % passive fiscal
lambd9F    =   [0 0                 0 0 1 1]; % active fiscal

% monetary policy weights six regimes
lambd1M	   =	[lambd1 lambd1 lambd1 lambd1 lambd1 lambd1];     % output 1
lambd3M    =	[lambd3 lambd3 lambd3 lambd3 lambd3 lambd3];     % output 2
lambd4M    =    [lambd4 lambd44 lambd4 lambd44 lambd4 lambd44];  % inflation stabilisation term 1   
lambd5M    =    [lambd5 lambd5 lambd5 lambd5 lambd5 lambd5];  % inflation stabilisation term 2
lambd6M    =	[lambd6 lambd6 lambd6 lambd6 lambd6 lambd6];     % intrest rate smoothing term  
lambd7M    =    [0 0 0 0 0 0];                                   % tax rate smoothing term

% fiscal policy transition probabilities
z11	=	0.875;     
z12	=	0.004;     
z13	=   1-z11-z12; 

z22	=	0.966;
z23	=	0.007;
z21 =   1-z22-z23;

z31	=	0.001;
z33	=	0.916; %0.996;  %This is to increase the probablity to say at AFMC
z32 =   1-z31-z33;

% monetary policy transition probabilities
q11	=	0.962;
q22	=	0.956;

% Markov Transition Matrix
strans = [z11     z21      z31
          z12     z22      z32
          z13     z23      z33];
				 		
strans2 = [q11  (1-q22)
          (1-q11) q22];
      
% here you can experiment to increase the credibility of stay the AF by
% increasing z33; 

% strans =  [1   0     z31
%            0   1     z32
%            0   0     0.996];
% 				 		
% strans2 = [1  0
%            0  1];
      
      
PP = kron(strans,strans2);  
Ps = PP'^10000;
ppp = Ps(1,:); 

% Data means      
rr     = 2;
p_i_ss = 2;
ga_mma = 0.46;
g_y    = 0.08;  
pmb_y  = 1.24;
pmb_ya = 1.24/4; 
tau    = 0.175;
tr     = 0.0919;

p11	=	0.965;
p22	=	0.894;

strans3 = [p11  (1-p22)
          (1-p11) p22];
      
PPs = strans3'^10000;      
pppp = PPs(1,:);  

%********************************************************************
%additional parameters 
%********************************************************************/ 
pibar = p_i_ss/400+1;
gam   = ga_mma/100+1;
beta  = 1/(1+rr/400); 
bet   = 1/(1+rr/400); 
rho   = (1-0.05)/beta;
Dinom = calvo*(1 + beta*index) + (1-calvo)*index; 
xf    = calvo/Dinom;
xb    = index/Dinom;
kc    = ((1-calvo)*(1-index)*(1-calvo*beta))/Dinom;

%---------------------------------------------------------------------
ns = 6;          %number of modes
nX = 17;         %predeterminate  
nxx = 8;         %jump
n = nX + nxx;    %total number of predeterminate and jump varaibles
p = 2;           %total number of instruments
nld = 1;         %number of leader
nfl = 1;         %number of follower
target  = 7;     %number of targeted variables of monetary (follower) 
target2 = 9;     %number of targeted variables of fiscal(leader)
shock   = 7;    %number of shocks
ny      = 7;     %number of variables
%---------------------------------------------------------------------

% variable indices
vi_Tr	=	1	;
vi_tech	=	2	;
vi_cost	=	3	;
vi_perf	=	4	;
vi_gsh	=	5	;
vi_taustarsh	=	6	;
vi_debtsh	=	7	;
vi_glag	=	8	;
vi_taulag	=	9	;
vi_debtlag	=	10	;
vi_ylag	=	11	;
vi_ilag	=	12	;
vi_clag	=	13	;
vi_pilag	=	14	;
vi_pmlag	=	15	;
vi_taulagstart	=	16	;
vi_taulagstart2	=	17	;
vi_taustart	=	18	;
vi_taustart2	=	19	;
vi_g	=	20	;
vi_debt	=	21	;
vi_y	=	22	;
vi_pm	=	23	;
vi_c	=	24	;
vi_pi	=	25	;
vi_tau	=	26	;
vi_i	=	27	;
%---------------------------------------------------------------------
% instrument 
in_tau  = 1;  
in_i    = 2;  
%---------------------------------------------------------------------
% Equation indices
eq_Tr	=	1	;
eq_tech	=	2	;
eq_cost	=	3	;
eq_perf	=	4	;
eq_gsh	=	5	;
eq_taustarsh	=	6	;
eq_debtsh	=	7	;
eq_glag	=	8	;
eq_taulag	=	9	;
eq_debtlag	=	10	;
eq_ylag	=	11	;
eq_ilag	=	12	;
eq_clag	=	13	;
eq_pilag	=	14	;
eq_pmlag	=	15	;
eq_taulagstart	=	16	;
eq_taulagstart2	=	17	;
eq_taustart	=	18	;
eq_taustart2 =	19	;
eq_g	=	20	;
eq_debt	=	21	;
eq_y	=	22	;
eq_pm	=	23	;
eq_c	=	24	;
eq_pi	=	25	; 

%---------------------------------------------------------------------
%initialize matrices 
Am = zeros(n,n,ns);
Bm = zeros(n,n,ns);
Cm = zeros(n,p,ns);
HH = zeros(nxx,nxx,ns);
lossld = zeros(target2,target2,ns); % fiscal 
lossfl = zeros(target,target,ns); % monetary

% two shocks regimes 
nss = 2; 
C  = zeros(nX,nX,nss);

for ii = 1:nss 
%%%**********************************************************
%%%  4 shocks 
%%%**********************************************************
C(vi_Tr,vi_Tr,ii)                 = s_Tr(ii);
C(vi_tech,vi_tech,ii)             = s_tech(ii);
C(vi_cost,vi_cost,ii)             = s_cost(ii); 
C(vi_perf,vi_perf,ii)             = s_perf(ii); 
C(vi_gsh,vi_gsh,ii)               = s_gov(ii); 
C(vi_taustarsh,vi_taustarsh,ii)   = s_taustar(ii); 
C(vi_debtsh,vi_debtsh,ii)         = s_debt(ii); 
end

CC = pppp(1,1)*C(:,:,1) + pppp(1,2)*C(:,:,2); %weighted average
%CC  = C(:,:,1); % low volatility
%CC = C(:,:,2); % high volatility

% The OF and PF regimes
for i = 1:4
% transfers 
Am(eq_Tr,vi_Tr,i) = 1;                  Bm(eq_Tr,vi_Tr,i) = rho_Tr;
% cost push 
Am(eq_cost,vi_cost,i) = 1;              Bm(eq_cost,vi_cost,i) = rho_cost;
% perference shocks
Am(eq_perf,vi_perf,i) = 1;              Bm(eq_perf,vi_perf,i) = rho_perf;
% technology
Am(eq_tech,vi_tech,i) = 1;              Bm(eq_tech,vi_tech,i) = rho_tech;
% government shocks
Am(eq_gsh,vi_gsh,i) = 1;                Bm(eq_gsh,vi_gsh,i) = 0.0;
% tau shocks
Am(eq_taustarsh,vi_taustarsh,i) = 1;    Bm(eq_taustarsh,vi_taustarsh,i) = 0.0;
% debt shocks
Am(eq_debtsh,vi_debtsh,i) = 1;          Bm(eq_debtsh,vi_debtsh,i) = 0.0;
%  passive tax rule 
Bm(eq_taustart,vi_taustart,i)     = -1;
Bm(eq_taustart,vi_taulagstart,i)  = rho_tau;
Bm(eq_taustart,vi_debtlag,i)      = (1-rho_tau)*debtp; 
Bm(eq_taustart,vi_y,i)            = (1-rho_tau)*taug_chi; 
Bm(eq_taustart,vi_g,i)            = (1-rho_tau)*yg_chi; 
Bm(eq_taustart,vi_taustarsh,i)    = 1; 
%  active tax rule 
Bm(eq_taustart2,vi_taustart2,i)     = -1;
Bm(eq_taustart2,vi_taulagstart2,i)  = rho_tau0;
Bm(eq_taustart2,vi_debtlag,i)       = (1-rho_tau0)*debtp0; 
Bm(eq_taustart2,vi_y,i)             = (1-rho_tau0)*taug_chi; 
Bm(eq_taustart2,vi_g,i)             = (1-rho_tau0)*yg_chi; 
Bm(eq_taustart2,vi_taustarsh,i)     = 1; 
% government spending rule 
Bm(eq_g,vi_g,i)       = -1; 
Bm(eq_g,vi_glag,i)    = rho_gov;
Bm(eq_g,vi_debtlag,i) = (1-rho_gov)*y_chi; 
Bm(eq_g,vi_gsh,i)     = 1;
% budget contraint
Bm(eq_debt,vi_debt,i)    =  -1;
Bm(eq_debt,vi_debtlag,i) =  1/beta;
Bm(eq_debt,vi_ylag,i)    =  pmb_y/beta;
Bm(eq_debt,vi_y,i)       = -pmb_y/beta; 
Bm(eq_debt,vi_pi,i)      = -pmb_y/beta;
Bm(eq_debt,vi_tech,i)    = -pmb_y/beta;
Bm(eq_debt,vi_Tr,i)      =  1;
Cm(eq_debt,in_tau,i)     = -1;
Bm(eq_debt,vi_g,i)       =  1;
Bm(eq_debt,vi_pm,i)      =  (pmb_y*rho)/(gam*pibar); 
Bm(eq_debt,vi_pmlag,i)   =  -pmb_y/beta;  
Bm(eq_debt,vi_debtsh,i) = 1;

%  output
Bm(eq_y,vi_y,i) = -1;
Bm(eq_y,vi_c,i) =  1;
Bm(eq_y,vi_g,i) =  1/(1-g_y);                

% linking equation = the number of jump variables
Am(eq_glag,vi_glag,i)        = 1;            Bm(eq_glag,vi_g,i)        = 1;
Am(eq_debtlag,vi_debtlag,i)  = 1;            Bm(eq_debtlag,vi_debt,i)  = 1;
Am(eq_ylag,vi_ylag,i)        = 1;            Bm(eq_ylag,vi_y,i)        = 1;
Am(eq_pmlag,vi_pmlag,i)      = 1;            Bm(eq_pmlag,vi_pm,i)      = 1;
Am(eq_pilag,vi_pilag,i)      = 1;            Bm(eq_pilag,vi_pi,i)      = 1;
Am(eq_clag,vi_clag,i)        = 1;            Bm(eq_clag,vi_c,i)        = 1;
Am(eq_taulagstart,vi_taulagstart,i)   = 1;   Bm(eq_taulagstart,vi_taustart,i)    = 1;
Am(eq_taulagstart2,vi_taulagstart2,i) = 1;   Bm(eq_taulagstart2,vi_taustart2,i)  = 1;

% instruments
Am(eq_taulag,vi_taulag,i) = 1;      Cm(eq_taulag,in_tau,i) = 1;
Am(eq_ilag,vi_ilag,i)    = 1;       Cm(eq_ilag,in_i,i)     = 1;

%  Consumption Euler Equation
Am(eq_c,vi_c,i)    =  -1;
Am(eq_c,vi_pi,i)   =  -(1-habit)/sigma; 
Bm(eq_c,vi_tech,i) =   rho_tech*(1-habit)/sigma;
Bm(eq_c,vi_c,i)    =  -(1+habit);
Bm(eq_c,vi_clag,i) =   habit;
Bm(eq_c,vi_perf,i) =  -(1-habit)*(1-rho_perf);
Cm(eq_c,in_i,i)    =  -(1-habit)/sigma;

% PC
Am(eq_pi,vi_pi,i)     = -xf*beta; 
Bm(eq_pi,vi_pi,i)     = -1;
Bm(eq_pi,vi_pilag,i)  =  xb;
Bm(eq_pi,vi_c,i)      = (kc*sigma)/(1-habit); 
Bm(eq_pi,vi_clag,i)   = -kc*sigma*habit/(1-habit); 
Bm(eq_pi,vi_y,i)      =  kc*phi;
Bm(eq_pi,vi_cost,i)   =  kc;
Cm(eq_pi,in_tau,i)    =  kc/(1-tau); 

% PM
Am(eq_pm,vi_pm,i) = rho*beta/(pibar*gam);  
Bm(eq_pm,vi_pm,i) = 1;
Cm(eq_pm,in_i,i)  = 1;

HH(:,:,i) = Am(nX+1:nX+nxx,nX+1:nX+nxx,i);
end

% The AF regime - I left these two regimes so you can change budget
% constraint. 

for i = 5:6
% transfers 
Am(eq_Tr,vi_Tr,i) = 1;                  Bm(eq_Tr,vi_Tr,i) = rho_Tr;
% cost push 
Am(eq_cost,vi_cost,i) = 1;              Bm(eq_cost,vi_cost,i) = rho_cost;
% perference shocks
Am(eq_perf,vi_perf,i) = 1;              Bm(eq_perf,vi_perf,i) = rho_perf;
% technology
Am(eq_tech,vi_tech,i) = 1;              Bm(eq_tech,vi_tech,i) = rho_tech;
% government shocks
Am(eq_gsh,vi_gsh,i) = 1;                Bm(eq_gsh,vi_gsh,i) = 0.0;
% tau shocks
Am(eq_taustarsh,vi_taustarsh,i) = 1;    Bm(eq_taustarsh,vi_taustarsh,i) = 0.0;
% debt shocks
Am(eq_debtsh,vi_debtsh,i) = 1;          Bm(eq_debtsh,vi_debtsh,i) = 0.0;
%  passive tax rule 
Bm(eq_taustart,vi_taustart,i)     = -1;
Bm(eq_taustart,vi_taulagstart,i)  = rho_tau;
Bm(eq_taustart,vi_debtlag,i)      = (1-rho_tau)*debtp; 
Bm(eq_taustart,vi_y,i)            = (1-rho_tau)*taug_chi; 
Bm(eq_taustart,vi_g,i)            = (1-rho_tau)*yg_chi; 
Bm(eq_taustart,vi_taustarsh,i)    = 1; 
%  active tax rule 
Bm(eq_taustart2,vi_taustart2,i)     = -1;
Bm(eq_taustart2,vi_taulagstart2,i)  = rho_tau0;
Bm(eq_taustart2,vi_debtlag,i)       = (1-rho_tau0)*debtp0; 
Bm(eq_taustart2,vi_y,i)             = (1-rho_tau0)*taug_chi; 
Bm(eq_taustart2,vi_g,i)             = (1-rho_tau0)*yg_chi; 
Bm(eq_taustart2,vi_taustarsh,i)     = 1; 
% government spending rule 
Bm(eq_g,vi_g,i)       = -1; 
Bm(eq_g,vi_glag,i)    = rho_gov;
Bm(eq_g,vi_debtlag,i) = (1-rho_gov)*y_chi; 
Bm(eq_g,vi_gsh,i)     = 1;
% budget contraint
Bm(eq_debt,vi_debt,i)    =  -1;
Bm(eq_debt,vi_debtlag,i) =  1/beta;
Bm(eq_debt,vi_ylag,i)    =  pmb_y/beta;
Bm(eq_debt,vi_y,i)       = -pmb_y/beta; 
Bm(eq_debt,vi_pi,i)      = -pmb_y/beta;
Bm(eq_debt,vi_tech,i)    = -pmb_y/beta;

Bm(eq_debt,vi_Tr,i)      =  1;
Cm(eq_debt,in_tau,i)     = -1; %switch back to instrument, comment out the line below.  
%Bm(eq_debt,vi_taustart2,i) = -1;
Bm(eq_debt,vi_g,i)       =  1;
Bm(eq_debt,vi_pm,i)      =  (pmb_y*rho)/(gam*pibar); 
Bm(eq_debt,vi_pmlag,i)   =  -pmb_y/beta;  
Bm(eq_debt,vi_debtsh,i) = 1;

%  output
Bm(eq_y,vi_y,i) = -1;
Bm(eq_y,vi_c,i) =  1;
Bm(eq_y,vi_g,i) =  1/(1-g_y);                

% linking equation = the number of jump variables
Am(eq_glag,vi_glag,i)        = 1;            Bm(eq_glag,vi_g,i)        = 1;
Am(eq_debtlag,vi_debtlag,i)  = 1;            Bm(eq_debtlag,vi_debt,i)  = 1;
Am(eq_ylag,vi_ylag,i)        = 1;            Bm(eq_ylag,vi_y,i)        = 1;
Am(eq_pmlag,vi_pmlag,i)      = 1;            Bm(eq_pmlag,vi_pm,i)      = 1;
Am(eq_pilag,vi_pilag,i)      = 1;            Bm(eq_pilag,vi_pi,i)      = 1;
Am(eq_clag,vi_clag,i)        = 1;            Bm(eq_clag,vi_c,i)        = 1;
Am(eq_taulagstart,vi_taulagstart,i)   = 1;   Bm(eq_taulagstart,vi_taustart,i)    = 1;
Am(eq_taulagstart2,vi_taulagstart2,i) = 1;   Bm(eq_taulagstart2,vi_taustart2,i)  = 1;

% instruments
Am(eq_taulag,vi_taulag,i) = 1;      Cm(eq_taulag,in_tau,i) = 1;
Am(eq_ilag,vi_ilag,i)    = 1;       Cm(eq_ilag,in_i,i)     = 1;

%  Consumption Euler Equation
Am(eq_c,vi_c,i)    =  -1;
Am(eq_c,vi_pi,i)   =  -(1-habit)/sigma; 
Bm(eq_c,vi_tech,i) =   rho_tech*(1-habit)/sigma;
Bm(eq_c,vi_c,i)    =  -(1+habit);
Bm(eq_c,vi_clag,i) =   habit;
Bm(eq_c,vi_perf,i) =  -(1-habit)*(1-rho_perf);
Cm(eq_c,in_i,i)    =  -(1-habit)/sigma;

% PC
Am(eq_pi,vi_pi,i)     = -xf*beta; 
Bm(eq_pi,vi_pi,i)     = -1;
Bm(eq_pi,vi_pilag,i)  =  xb;
Bm(eq_pi,vi_c,i)      = (kc*sigma)/(1-habit); 
Bm(eq_pi,vi_clag,i)   = -kc*sigma*habit/(1-habit); 
Bm(eq_pi,vi_y,i)      =  kc*phi;
Bm(eq_pi,vi_cost,i)   =  kc;
Cm(eq_pi,in_tau,i)    =  kc/(1-tau); 

% PM
Am(eq_pm,vi_pm,i) = rho*beta/(pibar*gam);  
Bm(eq_pm,vi_pm,i) = 1;
Cm(eq_pm,in_i,i)  = 1;

HH(:,:,i) = Am(nX+1:nX+nxx,nX+1:nX+nxx,i);
end

% We use the estimated weights  
df = zeros(target2,n+p,ns); %Fiscal policy 
d  = zeros(target,n+p,ns);  %Money policy

for i = 1:ns
    
   lossld(:,:,i) = diag([lambd1F(i) 0 lambd3F(i) lambd4F(i) lambd5F(i) 0 lambd7F(i)	lambd8F(i)  lambd9F(i)]); % fiscal policy
   
   df(1,vi_perf,i) = 1;    
   df(1,vi_c,i)   = 1/(1-habit); 
   df(1,vi_clag,i) = -habit/(1-habit);

   df(2,vi_perf,i) = 1; 
   df(2,vi_g,i)    = 1; 

   df(3,vi_y,i)   = 1;
   df(3,vi_perf,i) = -sigma/phi;

   df(4,vi_pi,i) = 1;

   df(5,vi_pi,i)    = 1;
   df(5,vi_pilag,i) = -1;

   df(6,vi_ilag,i) = -1;
   df(6,vi_i,i)    = 1;

   df(7,vi_tau,i)    =  1;
   df(7,vi_taulag,i) = -1;

   df(8,vi_tau,i)     =  1;
   df(8,vi_taustart,i) = -1;  %passive fiscal rule 

   df(9,vi_tau,i)      =  1;
   df(9,vi_taustart2,i) = -1; %active fiscal rule 
   
 lossfl(:,:,i) = diag([lambd1M(i) 0 lambd3M(i) lambd4M(i) lambd5M(i) lambd6M(i) 0 ]); % monetary policy
   
  d(1,vi_perf,i)   = 1; 
  d(1,vi_c,i)      = 1/(1-habit); 
  d(1,vi_clag,i)   = -habit/(1-habit);

  d(2,vi_perf,i)   = 1; 
  d(2,vi_g,i)      = 1; 

  d(3,vi_y,i)      = 1;
  d(3,vi_perf,i)   = -sigma/phi;

  d(4,vi_pi,i)     = 1;

  d(5,vi_pi,i)     =  1;
  d(5,vi_pilag,i)  = -1;

  d(6,vi_i,i)      =  1;
  d(6,vi_ilag,i)   = -1;

  d(7,vi_tau,i)    =  1;
  d(7,vi_taulag,i) = -1;

end

% welfare functions is here
for j = 1:ns   
   qhld(:,:,j) = df(:,:,j)'*lossld(:,:,j)*df(:,:,j); 
   qhfl(:,:,j) = d(:,:,j)' *lossfl(:,:,j)*d(:,:,j);
end

Qld  = qhld(1:nX+nxx,1:nX+nxx,:);  
Pld  = qhld(1:nX+nxx,nX+nxx+1:nX+nxx+p,:);
Rld  = qhld(nX+nxx+1:nX+nxx+p,nX+nxx+1:nX+nxx+p,:);

Qfl = qhfl(1:nX+nxx,1:nX+nxx,:); 
Pfl = qhfl(1:nX+nxx,nX+nxx+1:nX+nxx+p,:);
Rfl = qhfl(nX+nxx+1:nX+nxx+p,nX+nxx+1:nX+nxx+p,:);

Vflt1_1 = ones(nX,nX)*1;  Vflt1_2 = ones(nX,nX)*1; Vflt1_3 = ones(nX,nX)*1;
Vflt1_4 = ones(nX,nX)*1;  Vflt1_5 = ones(nX,nX)*1; Vflt1_6 = ones(nX,nX)*1;

Vldt1_1 = ones(nX,nX)*1;  Vldt1_2 = ones(nX,nX)*1; Vldt1_3 = ones(nX,nX)*1;
Vldt1_4 = ones(nX,nX)*1;  Vldt1_5 = ones(nX,nX)*1; Vldt1_6 = ones(nX,nX)*1;

Ct1_1 = ones(nxx,nX)*1;   Ct1_2 = ones(nxx,nX)*1; Ct1_3 = ones(nxx,nX)*1;
Ct1_4 = ones(nxx,nX)*1;   Ct1_5 = ones(nxx,nX)*1; Ct1_6 = ones(nxx,nX)*1;

%% the following are the leadership code with transition probabilities

% initialise values
indexx = 0;  indicator = 0;
n = nX + nxx; 
k = nld + nfl; 

Fld_1 = ones(nld,nX)*2;  Fld_2 = ones(nld,nX)*2;  Fld_3 = ones(nld,nX)*2;  Fld_4 = ones(nld,nX)*2;  Fld_5 = ones(nld,nX)*2;  Fld_6 = ones(nld,nX)*2; %leader's X
Ffl_1 = ones(nfl,nX)*2;  Ffl_2 = ones(nfl,nX)*2;  Ffl_3 = ones(nfl,nX)*2;  Ffl_4 = ones(nfl,nX)*2;  Ffl_5 = ones(nfl,nX)*2;  Ffl_6 = ones(nfl,nX)*2; %follower's X
Lfl_1 = ones(nfl,nld)*2; Lfl_2 = ones(nfl,nld)*2; Lfl_3 = ones(nfl,nld)*2; Lfl_4 = ones(nfl,nld)*2; Lfl_5 = ones(nfl,nld)*2; Lfl_6 = ones(nfl,nld)*2; %follower's uL

% select matrics 
Qld11_1 = Qld(1:nX,1:nX,1);   Qld12_1 = Qld(1:nX,nX+1:n,1);    Qld21_1 = Qld(nX+1:n,1:nX,1);    Qld22_1 = Qld(nX+1:n,nX+1:n,1); % stage_1
Qld11_2 = Qld(1:nX,1:nX,2);   Qld12_2 = Qld(1:nX,nX+1:n,2);    Qld21_2 = Qld(nX+1:n,1:nX,2);    Qld22_2 = Qld(nX+1:n,nX+1:n,2); % stage_1
Qld11_3 = Qld(1:nX,1:nX,3);   Qld12_3 = Qld(1:nX,nX+1:n,3);    Qld21_3 = Qld(nX+1:n,1:nX,3);    Qld22_3 = Qld(nX+1:n,nX+1:n,3); % stage_2
Qld11_4 = Qld(1:nX,1:nX,4);   Qld12_4 = Qld(1:nX,nX+1:n,4);    Qld21_4 = Qld(nX+1:n,1:nX,4);    Qld22_4 = Qld(nX+1:n,nX+1:n,4); % stage_2
Qld11_5 = Qld(1:nX,1:nX,5);   Qld12_5 = Qld(1:nX,nX+1:n,5);    Qld21_5 = Qld(nX+1:n,1:nX,5);    Qld22_5 = Qld(nX+1:n,nX+1:n,5); % stage_3
Qld11_6 = Qld(1:nX,1:nX,6);   Qld12_6 = Qld(1:nX,nX+1:n,6);    Qld21_6 = Qld(nX+1:n,1:nX,6);    Qld22_6 = Qld(nX+1:n,nX+1:n,6); % stage_3

Rld11_1 = Rld(1:nld,1:nld,1);  Rld12_1 = Rld(1:nld,nld+1:k,1);  Rld21_1 = Rld(nld+1:k,1:nld,1);  Rld22_1 = Rld(nld+1:k,nld+1:k,1);
Rld11_2 = Rld(1:nld,1:nld,2);  Rld12_2 = Rld(1:nld,nld+1:k,2);  Rld21_2 = Rld(nld+1:k,1:nld,2);  Rld22_2 = Rld(nld+1:k,nld+1:k,2);
Rld11_3 = Rld(1:nld,1:nld,3);  Rld12_3 = Rld(1:nld,nld+1:k,3);  Rld21_3 = Rld(nld+1:k,1:nld,3);  Rld22_3 = Rld(nld+1:k,nld+1:k,3);
Rld11_4 = Rld(1:nld,1:nld,4);  Rld12_4 = Rld(1:nld,nld+1:k,4);  Rld21_4 = Rld(nld+1:k,1:nld,4);  Rld22_4 = Rld(nld+1:k,nld+1:k,4);
Rld11_5 = Rld(1:nld,1:nld,5);  Rld12_5 = Rld(1:nld,nld+1:k,5);  Rld21_5 = Rld(nld+1:k,1:nld,5);  Rld22_5 = Rld(nld+1:k,nld+1:k,5);
Rld11_6 = Rld(1:nld,1:nld,6);  Rld12_6 = Rld(1:nld,nld+1:k,6);  Rld21_6 = Rld(nld+1:k,1:nld,6);  Rld22_6 = Rld(nld+1:k,nld+1:k,6);

Pld11_1 = Pld(1:nX,1:nld,1);   Pld12_1 = Pld(1:nX,nld+1:k,1);   Pld21_1 = Pld(nX+1:n,1:nld,1);   Pld22_1 = Pld(nX+1:n,nld+1:k,1);
Pld11_2 = Pld(1:nX,1:nld,2);   Pld12_2 = Pld(1:nX,nld+1:k,2);   Pld21_2 = Pld(nX+1:n,1:nld,2);   Pld22_2 = Pld(nX+1:n,nld+1:k,2);
Pld11_3 = Pld(1:nX,1:nld,3);   Pld12_3 = Pld(1:nX,nld+1:k,3);   Pld21_3 = Pld(nX+1:n,1:nld,3);   Pld22_3 = Pld(nX+1:n,nld+1:k,3);
Pld11_4 = Pld(1:nX,1:nld,4);   Pld12_4 = Pld(1:nX,nld+1:k,4);   Pld21_4 = Pld(nX+1:n,1:nld,4);   Pld22_4 = Pld(nX+1:n,nld+1:k,4);
Pld11_5 = Pld(1:nX,1:nld,5);   Pld12_5 = Pld(1:nX,nld+1:k,5);   Pld21_5 = Pld(nX+1:n,1:nld,5);   Pld22_5 = Pld(nX+1:n,nld+1:k,5);
Pld11_6 = Pld(1:nX,1:nld,6);   Pld12_6 = Pld(1:nX,nld+1:k,6);   Pld21_6 = Pld(nX+1:n,1:nld,6);   Pld22_6 = Pld(nX+1:n,nld+1:k,6);

Qfl11_1 = Qfl(1:nX,1:nX,1);   Qfl12_1 = Qfl(1:nX,nX+1:n,1);   Qfl21_1 = Qfl(nX+1:n,1:nX,1);     Qfl22_1 = Qfl(nX+1:n,nX+1:n,1); 
Qfl11_2 = Qfl(1:nX,1:nX,2);   Qfl12_2 = Qfl(1:nX,nX+1:n,2);   Qfl21_2 = Qfl(nX+1:n,1:nX,2);     Qfl22_2 = Qfl(nX+1:n,nX+1:n,2); 
Qfl11_3 = Qfl(1:nX,1:nX,3);   Qfl12_3 = Qfl(1:nX,nX+1:n,3);   Qfl21_3 = Qfl(nX+1:n,1:nX,3);     Qfl22_3 = Qfl(nX+1:n,nX+1:n,3); 
Qfl11_4 = Qfl(1:nX,1:nX,4);   Qfl12_4 = Qfl(1:nX,nX+1:n,4);   Qfl21_4 = Qfl(nX+1:n,1:nX,4);     Qfl22_4 = Qfl(nX+1:n,nX+1:n,4); 
Qfl11_5 = Qfl(1:nX,1:nX,5);   Qfl12_5 = Qfl(1:nX,nX+1:n,5);   Qfl21_5 = Qfl(nX+1:n,1:nX,5);     Qfl22_5 = Qfl(nX+1:n,nX+1:n,5); 
Qfl11_6 = Qfl(1:nX,1:nX,6);   Qfl12_6 = Qfl(1:nX,nX+1:n,6);   Qfl21_6 = Qfl(nX+1:n,1:nX,6);     Qfl22_6 = Qfl(nX+1:n,nX+1:n,6); 

Rfl11_1 = Rfl(1:nld,1:nld,1);  Rfl12_1 = Rfl(1:nld,nld+1:k,1);  Rfl21_1 = Rfl(nld+1:k,1:nld,1);  Rfl22_1 = Rfl(nld+1:k,nld+1:k,1);
Rfl11_2 = Rfl(1:nld,1:nld,2);  Rfl12_2 = Rfl(1:nld,nld+1:k,2);  Rfl21_2 = Rfl(nld+1:k,1:nld,2);  Rfl22_2 = Rfl(nld+1:k,nld+1:k,2);
Rfl11_3 = Rfl(1:nld,1:nld,3);  Rfl12_3 = Rfl(1:nld,nld+1:k,3);  Rfl21_3 = Rfl(nld+1:k,1:nld,3);  Rfl22_3 = Rfl(nld+1:k,nld+1:k,3);
Rfl11_4 = Rfl(1:nld,1:nld,4);  Rfl12_4 = Rfl(1:nld,nld+1:k,4);  Rfl21_4 = Rfl(nld+1:k,1:nld,4);  Rfl22_4 = Rfl(nld+1:k,nld+1:k,4);
Rfl11_5 = Rfl(1:nld,1:nld,5);  Rfl12_5 = Rfl(1:nld,nld+1:k,5);  Rfl21_5 = Rfl(nld+1:k,1:nld,5);  Rfl22_5 = Rfl(nld+1:k,nld+1:k,5);
Rfl11_6 = Rfl(1:nld,1:nld,6);  Rfl12_6 = Rfl(1:nld,nld+1:k,6);  Rfl21_6 = Rfl(nld+1:k,1:nld,6);  Rfl22_6 = Rfl(nld+1:k,nld+1:k,6);

Pfl11_1 = Pfl(1:nX,1:nld,1);  Pfl12_1 = Pfl(1:nX,nld+1:k,1);  Pfl21_1 = Pfl(nX+1:n,1:nld,1);  Pfl22_1 = Pfl(nX+1:n,nld+1:k,1);
Pfl11_2 = Pfl(1:nX,1:nld,2);  Pfl12_2 = Pfl(1:nX,nld+1:k,2);  Pfl21_2 = Pfl(nX+1:n,1:nld,2);  Pfl22_2 = Pfl(nX+1:n,nld+1:k,2);
Pfl11_3 = Pfl(1:nX,1:nld,3);  Pfl12_3 = Pfl(1:nX,nld+1:k,3);  Pfl21_3 = Pfl(nX+1:n,1:nld,3);  Pfl22_3 = Pfl(nX+1:n,nld+1:k,3);
Pfl11_4 = Pfl(1:nX,1:nld,4);  Pfl12_4 = Pfl(1:nX,nld+1:k,4);  Pfl21_4 = Pfl(nX+1:n,1:nld,4);  Pfl22_4 = Pfl(nX+1:n,nld+1:k,4);
Pfl11_5 = Pfl(1:nX,1:nld,5);  Pfl12_5 = Pfl(1:nX,nld+1:k,5);  Pfl21_5 = Pfl(nX+1:n,1:nld,5);  Pfl22_5 = Pfl(nX+1:n,nld+1:k,5);
Pfl11_6 = Pfl(1:nX,1:nld,6);  Pfl12_6 = Pfl(1:nX,nld+1:k,6);  Pfl21_6 = Pfl(nX+1:n,1:nld,6);  Pfl22_6 = Pfl(nX+1:n,nld+1:k,6);

H_1 = HH(:,:,1); H_2 = HH(:,:,2);   H_3 = HH(:,:,3);  H_4 = HH(:,:,4);  H_5 = HH(:,:,5);  H_6 = HH(:,:,6);

A11_1 = Bm(1:nX,1:nX,1);     A11_2 = Bm(1:nX,1:nX,2);      A11_3 = Bm(1:nX,1:nX,3);      A11_4 = Bm(1:nX,1:nX,4);      A11_5 = Bm(1:nX,1:nX,5);     A11_6 = Bm(1:nX,1:nX,6);
A12_1 = Bm(1:nX,nX+1:n,1);   A12_2 = Bm(1:nX,nX+1:n,2);    A12_3 = Bm(1:nX,nX+1:n,3);    A12_4 = Bm(1:nX,nX+1:n,4);    A12_5 = Bm(1:nX,nX+1:n,5);   A12_6 = Bm(1:nX,nX+1:n,6);
A21_1 = Bm(nX+1:n,1:nX,1);   A21_2 = Bm(nX+1:n,1:nX,2);    A21_3 = Bm(nX+1:n,1:nX,3);    A21_4 = Bm(nX+1:n,1:nX,4);    A21_5 = Bm(nX+1:n,1:nX,5);   A21_6 = Bm(nX+1:n,1:nX,6);
A22_1 = Bm(nX+1:n,nX+1:n,1); A22_2 = Bm(nX+1:n,nX+1:n,2);  A22_3 = Bm(nX+1:n,nX+1:n,3);  A22_4 = Bm(nX+1:n,nX+1:n,4);  A22_5 = Bm(nX+1:n,nX+1:n,5); A22_6 = Bm(nX+1:n,nX+1:n,6);

B11_1 = Cm(1:nX,1:nld,1);          B11_2 = Cm(1:nX,1:nld,2);          B11_3 = Cm(1:nX,1:nld,3);          B11_4 = Cm(1:nX,1:nld,4);          B11_5 = Cm(1:nX,1:nld,5);          B11_6 = Cm(1:nX,1:nld,6); 
B12_1 = Cm(1:nX,(nld+1):k,1);      B12_2 = Cm(1:nX,(nld+1):k,2);      B12_3 = Cm(1:nX,(nld+1):k,3);      B12_4 = Cm(1:nX,(nld+1):k,4);      B12_5 = Cm(1:nX,(nld+1):k,5);      B12_6 = Cm(1:nX,(nld+1):k,6);
B21_1 = Cm((nX+1):n,1:nld,1);      B21_2 = Cm((nX+1):n,1:nld,2);      B21_3 = Cm((nX+1):n,1:nld,3);      B21_4 = Cm((nX+1):n,1:nld,4);      B21_5 = Cm((nX+1):n,1:nld,5);      B21_6 = Cm((nX+1):n,1:nld,6);
B22_1 = Cm((nX+1):n,(nld+1):k,1);  B22_2 = Cm((nX+1):n,(nld+1):k,2);  B22_3 = Cm((nX+1):n,(nld+1):k,3);  B22_4 = Cm((nX+1):n,(nld+1):k,4);  B22_5 = Cm((nX+1):n,(nld+1):k,5);  B22_6 = Cm((nX+1):n,(nld+1):k,6);

iter = 1; tol=1e-12; maxiter=10000; step = 0.7;
while (xxx>tol)&&(iter<maxiter) 
    
d1mat_1 = inv(A22_1 + (PP(1,1)*H_1*Ct1_1*A12_1 + PP(2,1)*H_2*Ct1_2*A12_2 + PP(3,1)*H_3*Ct1_3*A12_3 + PP(4,1)*H_4*Ct1_4*A12_4 + PP(5,1)*H_5*Ct1_5*A12_5 + PP(6,1)*H_6*Ct1_6*A12_6)); 
d1mat_2 = inv(A22_2 + (PP(1,2)*H_1*Ct1_1*A12_1 + PP(2,2)*H_2*Ct1_2*A12_2 + PP(3,2)*H_3*Ct1_3*A12_3 + PP(4,2)*H_4*Ct1_4*A12_4 + PP(5,2)*H_5*Ct1_5*A12_5 + PP(6,2)*H_6*Ct1_6*A12_6));
d1mat_3 = inv(A22_3 + (PP(1,3)*H_1*Ct1_1*A12_1 + PP(2,3)*H_2*Ct1_2*A12_2 + PP(3,3)*H_3*Ct1_3*A12_3 + PP(4,3)*H_4*Ct1_4*A12_4 + PP(5,3)*H_5*Ct1_5*A12_5 + PP(6,3)*H_6*Ct1_6*A12_6));
d1mat_4 = inv(A22_4 + (PP(1,4)*H_1*Ct1_1*A12_1 + PP(2,4)*H_2*Ct1_2*A12_2 + PP(3,4)*H_3*Ct1_3*A12_3 + PP(4,4)*H_4*Ct1_4*A12_4 + PP(5,4)*H_5*Ct1_5*A12_5 + PP(6,4)*H_6*Ct1_6*A12_6));
d1mat_5 = inv(A22_5 + (PP(1,5)*H_1*Ct1_1*A12_1 + PP(2,5)*H_2*Ct1_2*A12_2 + PP(3,5)*H_3*Ct1_3*A12_3 + PP(4,5)*H_4*Ct1_4*A12_4 + PP(5,5)*H_5*Ct1_5*A12_5 + PP(6,5)*H_6*Ct1_6*A12_6));
d1mat_6 = inv(A22_6 + (PP(1,6)*H_1*Ct1_1*A12_1 + PP(2,6)*H_2*Ct1_2*A12_2 + PP(3,6)*H_3*Ct1_3*A12_3 + PP(4,6)*H_4*Ct1_4*A12_4 + PP(5,6)*H_5*Ct1_5*A12_5 + PP(6,6)*H_6*Ct1_6*A12_6)); 

J_1 = d1mat_1 *((PP(1,1)*H_1*Ct1_1*A11_1 + PP(2,1)*H_2*Ct1_2*A11_2 + PP(3,1)*H_3*Ct1_3*A11_3 + PP(4,1)*H_4*Ct1_4*A11_4 + PP(5,1)*H_5*Ct1_5*A11_5 + PP(6,1)*H_6*Ct1_6*A11_6) + A21_1); 
J_2 = d1mat_2 *((PP(1,2)*H_1*Ct1_1*A11_1 + PP(2,2)*H_2*Ct1_2*A11_2 + PP(3,2)*H_3*Ct1_3*A11_3 + PP(4,2)*H_4*Ct1_4*A11_4 + PP(5,2)*H_5*Ct1_5*A11_5 + PP(6,2)*H_6*Ct1_6*A11_6) + A21_2);
J_3 = d1mat_3 *((PP(1,3)*H_1*Ct1_1*A11_1 + PP(2,3)*H_2*Ct1_2*A11_2 + PP(3,3)*H_3*Ct1_3*A11_3 + PP(4,3)*H_4*Ct1_4*A11_4 + PP(5,3)*H_5*Ct1_5*A11_5 + PP(6,3)*H_6*Ct1_6*A11_6) + A21_3);
J_4 = d1mat_4 *((PP(1,4)*H_1*Ct1_1*A11_1 + PP(2,4)*H_2*Ct1_2*A11_2 + PP(3,4)*H_3*Ct1_3*A11_3 + PP(4,4)*H_4*Ct1_4*A11_4 + PP(5,4)*H_5*Ct1_5*A11_5 + PP(6,4)*H_6*Ct1_6*A11_6) + A21_4);
J_5 = d1mat_5 *((PP(1,5)*H_1*Ct1_1*A11_1 + PP(2,5)*H_2*Ct1_2*A11_2 + PP(3,5)*H_3*Ct1_3*A11_3 + PP(4,5)*H_4*Ct1_4*A11_4 + PP(5,5)*H_5*Ct1_5*A11_5 + PP(6,5)*H_6*Ct1_6*A11_6) + A21_5);
J_6 = d1mat_6 *((PP(1,6)*H_1*Ct1_1*A11_1 + PP(2,6)*H_2*Ct1_2*A11_2 + PP(3,6)*H_3*Ct1_3*A11_3 + PP(4,6)*H_4*Ct1_4*A11_4 + PP(5,6)*H_5*Ct1_5*A11_5 + PP(6,6)*H_6*Ct1_6*A11_6) + A21_6);

Kld_1 = d1mat_1 *((PP(1,1)*H_1*Ct1_1*B11_1 + PP(2,1)*H_2*Ct1_2*B11_2 + PP(3,1)*H_3*Ct1_3*B11_3 + PP(4,1)*H_4*Ct1_4*B11_4 + PP(5,1)*H_5*Ct1_5*B11_5 + PP(6,1)*H_6*Ct1_6*B11_6) + B21_1);
Kld_2 = d1mat_2 *((PP(1,2)*H_1*Ct1_1*B11_1 + PP(2,2)*H_2*Ct1_2*B11_2 + PP(3,2)*H_3*Ct1_3*B11_3 + PP(4,2)*H_4*Ct1_4*B11_4 + PP(5,2)*H_5*Ct1_5*B11_5 + PP(6,2)*H_6*Ct1_6*B11_6) + B21_2);
Kld_3 = d1mat_3 *((PP(1,3)*H_1*Ct1_1*B11_1 + PP(2,3)*H_2*Ct1_2*B11_2 + PP(3,3)*H_3*Ct1_3*B11_3 + PP(4,3)*H_4*Ct1_4*B11_4 + PP(5,3)*H_5*Ct1_5*B11_5 + PP(6,3)*H_6*Ct1_6*B11_6) + B21_3);
Kld_4 = d1mat_4 *((PP(1,4)*H_1*Ct1_1*B11_1 + PP(2,4)*H_2*Ct1_2*B11_2 + PP(3,4)*H_3*Ct1_3*B11_3 + PP(4,4)*H_4*Ct1_4*B11_4 + PP(5,4)*H_5*Ct1_5*B11_5 + PP(6,4)*H_6*Ct1_6*B11_6) + B21_4);
Kld_5 = d1mat_5 *((PP(1,5)*H_1*Ct1_1*B11_1 + PP(2,5)*H_2*Ct1_2*B11_2 + PP(3,5)*H_3*Ct1_3*B11_3 + PP(4,5)*H_4*Ct1_4*B11_4 + PP(5,5)*H_5*Ct1_5*B11_5 + PP(6,5)*H_6*Ct1_6*B11_6) + B21_5);
Kld_6 = d1mat_6 *((PP(1,6)*H_1*Ct1_1*B11_1 + PP(2,6)*H_2*Ct1_2*B11_2 + PP(3,6)*H_3*Ct1_3*B11_3 + PP(4,6)*H_4*Ct1_4*B11_4 + PP(5,6)*H_5*Ct1_5*B11_5 + PP(6,6)*H_6*Ct1_6*B11_6) + B21_6);

Kfl_1 = d1mat_1 *((PP(1,1)*H_1*Ct1_1*B12_1 + PP(2,1)*H_2*Ct1_2*B12_2 + PP(3,1)*H_3*Ct1_3*B12_3 + PP(4,1)*H_4*Ct1_4*B12_4 + PP(5,1)*H_5*Ct1_5*B12_5 + PP(6,1)*H_6*Ct1_6*B12_6) + B22_1); 
Kfl_2 = d1mat_2 *((PP(1,2)*H_1*Ct1_1*B12_1 + PP(2,2)*H_2*Ct1_2*B12_2 + PP(3,2)*H_3*Ct1_3*B12_3 + PP(4,2)*H_4*Ct1_4*B12_4 + PP(5,2)*H_5*Ct1_5*B12_5 + PP(6,2)*H_6*Ct1_6*B12_6) + B22_2); 
Kfl_3 = d1mat_3 *((PP(1,3)*H_1*Ct1_1*B12_1 + PP(2,3)*H_2*Ct1_2*B12_2 + PP(3,3)*H_3*Ct1_3*B12_3 + PP(4,3)*H_4*Ct1_4*B12_4 + PP(5,3)*H_5*Ct1_5*B12_5 + PP(6,3)*H_6*Ct1_6*B12_6) + B22_3); 
Kfl_4 = d1mat_4 *((PP(1,4)*H_1*Ct1_1*B12_1 + PP(2,4)*H_2*Ct1_2*B12_2 + PP(3,4)*H_3*Ct1_3*B12_3 + PP(4,4)*H_4*Ct1_4*B12_4 + PP(5,4)*H_5*Ct1_5*B12_5 + PP(6,4)*H_6*Ct1_6*B12_6) + B22_4); 
Kfl_5 = d1mat_5 *((PP(1,5)*H_1*Ct1_1*B12_1 + PP(2,5)*H_2*Ct1_2*B12_2 + PP(3,5)*H_3*Ct1_3*B12_3 + PP(4,5)*H_4*Ct1_4*B12_4 + PP(5,5)*H_5*Ct1_5*B12_5 + PP(6,5)*H_6*Ct1_6*B12_6) + B22_5); 
Kfl_6 = d1mat_6 *((PP(1,6)*H_1*Ct1_1*B12_1 + PP(2,6)*H_2*Ct1_2*B12_2 + PP(3,6)*H_3*Ct1_3*B12_3 + PP(4,6)*H_4*Ct1_4*B12_4 + PP(5,6)*H_5*Ct1_5*B12_5 + PP(6,6)*H_6*Ct1_6*B12_6) + B22_6);
   
Aj_11 = A11_1 - A12_1 * J_1;
Aj_12 = A11_2 - A12_2 * J_1;
Aj_13 = A11_3 - A12_3 * J_1; 
Aj_14 = A11_4 - A12_4 * J_1;
Aj_15 = A11_5 - A12_5 * J_1;
Aj_16 = A11_6 - A12_6 * J_1;

Aj_21 = A11_1 - A12_1 * J_2;
Aj_22 = A11_2 - A12_2 * J_2;
Aj_23 = A11_3 - A12_3 * J_2;
Aj_24 = A11_4 - A12_4 * J_2;
Aj_25 = A11_5 - A12_5 * J_2;
Aj_26 = A11_6 - A12_6 * J_2;

Aj_31 = A11_1 - A12_1 * J_3;
Aj_32 = A11_2 - A12_2 * J_3;
Aj_33 = A11_3 - A12_3 * J_3;
Aj_34 = A11_4 - A12_4 * J_3;
Aj_35 = A11_5 - A12_5 * J_3;
Aj_36 = A11_6 - A12_6 * J_3;

Aj_41 = A11_1 - A12_1 * J_4;
Aj_42 = A11_2 - A12_2 * J_4;
Aj_43 = A11_3 - A12_3 * J_4;
Aj_44 = A11_4 - A12_4 * J_4;
Aj_45 = A11_5 - A12_5 * J_4;
Aj_46 = A11_6 - A12_6 * J_4;

Aj_51 = A11_1 - A12_1 * J_5;
Aj_52 = A11_2 - A12_2 * J_5;
Aj_53 = A11_3 - A12_3 * J_5;
Aj_54 = A11_4 - A12_4 * J_5;
Aj_55 = A11_5 - A12_5 * J_5;
Aj_56 = A11_6 - A12_6 * J_5;

Aj_61 = A11_1 - A12_1 * J_6;
Aj_62 = A11_2 - A12_2 * J_6;
Aj_63 = A11_3 - A12_3 * J_6;
Aj_64 = A11_4 - A12_4 * J_6;
Aj_65 = A11_5 - A12_5 * J_6;
Aj_66 = A11_6 - A12_6 * J_6; 

%/* eq13 */
Bld_11 = B11_1 - A12_1 *Kld_1;
Bld_12 = B11_2 - A12_2 *Kld_1; 
Bld_13 = B11_3 - A12_3 *Kld_1;
Bld_14 = B11_4 - A12_4 *Kld_1;
Bld_15 = B11_5 - A12_5 *Kld_1;
Bld_16 = B11_6 - A12_6 *Kld_1;

Bld_21 = B11_1 - A12_1 *Kld_2;
Bld_22 = B11_2 - A12_2 *Kld_2;
Bld_23 = B11_3 - A12_3 *Kld_2;
Bld_24 = B11_4 - A12_4 *Kld_2;
Bld_25 = B11_5 - A12_5 *Kld_2;
Bld_26 = B11_6 - A12_6 *Kld_2;

Bld_31 = B11_1 - A12_1 *Kld_3;
Bld_32 = B11_2 - A12_2 *Kld_3;
Bld_33 = B11_3 - A12_3 *Kld_3;
Bld_34 = B11_4 - A12_4 *Kld_3;
Bld_35 = B11_5 - A12_5 *Kld_3;
Bld_36 = B11_6 - A12_6 *Kld_3;

Bld_41 = B11_1 - A12_1 *Kld_4;
Bld_42 = B11_2 - A12_2 *Kld_4;
Bld_43 = B11_3 - A12_3 *Kld_4;
Bld_44 = B11_4 - A12_4 *Kld_4;
Bld_45 = B11_5 - A12_5 *Kld_4;
Bld_46 = B11_6 - A12_6 *Kld_4;

Bld_51 = B11_1 - A12_1 *Kld_5;
Bld_52 = B11_2 - A12_2 *Kld_5;
Bld_53 = B11_3 - A12_3 *Kld_5;
Bld_54 = B11_4 - A12_4 *Kld_5;
Bld_55 = B11_5 - A12_5 *Kld_5;
Bld_56 = B11_6 - A12_6 *Kld_5;

Bld_61 = B11_1 - A12_1 *Kld_6;
Bld_62 = B11_2 - A12_2 *Kld_6;
Bld_63 = B11_3 - A12_3 *Kld_6;
Bld_64 = B11_4 - A12_4 *Kld_6;
Bld_65 = B11_5 - A12_5 *Kld_6;
Bld_66 = B11_6 - A12_6 *Kld_6;

%/* eq14 */
Bfl_11 = B12_1 - A12_1 *Kfl_1;
Bfl_12 = B12_2 - A12_2 *Kfl_1;
Bfl_13 = B12_3 - A12_3 *Kfl_1;
Bfl_14 = B12_4 - A12_4 *Kfl_1;
Bfl_15 = B12_5 - A12_5 *Kfl_1;
Bfl_16 = B12_6 - A12_6 *Kfl_1;

Bfl_21 = B12_1 - A12_1 *Kfl_2;
Bfl_22 = B12_2 - A12_2 *Kfl_2;
Bfl_23 = B12_3 - A12_3 *Kfl_2;
Bfl_24 = B12_4 - A12_4 *Kfl_2;
Bfl_25 = B12_5 - A12_5 *Kfl_2;
Bfl_26 = B12_6 - A12_6 *Kfl_2;

Bfl_31 = B12_1 - A12_1 *Kfl_3;
Bfl_32 = B12_2 - A12_2 *Kfl_3;
Bfl_33 = B12_3 - A12_3 *Kfl_3;
Bfl_34 = B12_4 - A12_4 *Kfl_3;
Bfl_35 = B12_5 - A12_5 *Kfl_3;
Bfl_36 = B12_6 - A12_6 *Kfl_3;

Bfl_41 = B12_1 - A12_1 *Kfl_4;
Bfl_42 = B12_2 - A12_2 *Kfl_4;
Bfl_43 = B12_3 - A12_3 *Kfl_4;
Bfl_44 = B12_4 - A12_4 *Kfl_4;
Bfl_45 = B12_5 - A12_5 *Kfl_4;
Bfl_46 = B12_6 - A12_6 *Kfl_4;

Bfl_51 = B12_1 - A12_1 *Kfl_5;
Bfl_52 = B12_2 - A12_2 *Kfl_5;
Bfl_53 = B12_3 - A12_3 *Kfl_5;
Bfl_54 = B12_4 - A12_4 *Kfl_5;
Bfl_55 = B12_5 - A12_5 *Kfl_5;
Bfl_56 = B12_6 - A12_6 *Kfl_5;

Bfl_61 = B12_1 - A12_1 *Kfl_6;
Bfl_62 = B12_2 - A12_2 *Kfl_6;
Bfl_63 = B12_3 - A12_3 *Kfl_6;
Bfl_64 = B12_4 - A12_4 *Kfl_6;
Bfl_65 = B12_5 - A12_5 *Kfl_6;
Bfl_66 = B12_6 - A12_6 *Kfl_6;  

Qfls_1   = Qfl11_1 - Qfl12_1*J_1 - J_1'*Qfl21_1 + J_1'*Qfl22_1*J_1;   
Ufls_1   = J_1'*Qfl22_1*Kfl_1 - Qfl12_1*Kfl_1 + Pfl12_1-J_1'*Pfl22_1;
Ulds_1   = J_1'*Qfl22_1*Kld_1 - Qfl12_1*Kld_1 + Pfl11_1-J_1'*Pfl21_1;        
Rfls_1   = Rfl22_1 + Kfl_1'*Qfl22_1*Kfl_1 - Pfl22_1'*Kfl_1 - Kfl_1'*Pfl22_1; 
Pfls_1   = Kld_1'*Qfl22_1*Kfl_1 - Kld_1'*Pfl22_1 + Rfl12_1 - Pfl21_1'*Kfl_1; 
Tfls_1   = Kld_1'*Qfl22_1*Kld_1 - Kld_1'*Pfl21_1 - Pfl21_1'*Kld_1 + Rfl11_1; 

Qfls_2 = Qfl11_2 - Qfl12_2*J_2 - J_2'*Qfl21_2 + J_2'*Qfl22_2*J_2;
Ufls_2 = J_2'*Qfl22_2*Kfl_2 - Qfl12_2*Kfl_2 + Pfl12_2-J_2'*Pfl22_2;
Ulds_2 = J_2'*Qfl22_2*Kld_2 - Qfl12_2*Kld_2 + Pfl11_2-J_2'*Pfl21_2;
Rfls_2 = Rfl22_2 + Kfl_2'*Qfl22_2*Kfl_2 - Pfl22_2'*Kfl_2 - Kfl_2'*Pfl22_2;
Pfls_2 = Kld_2'*Qfl22_2*Kfl_2 - Kld_2'*Pfl22_2 + Rfl12_2 - Pfl21_2'*Kfl_2;
Tfls_2 = Kld_2'*Qfl22_2*Kld_2 - Kld_2'*Pfl21_2 - Pfl21_2'*Kld_2 + Rfl11_2;

Qfls_3 = Qfl11_3 - Qfl12_3*J_3 - J_3'*Qfl21_3 + J_3'*Qfl22_3*J_3;
Ufls_3 = J_3'*Qfl22_3*Kfl_3 - Qfl12_3*Kfl_3 + Pfl12_3-J_3'*Pfl22_3;
Ulds_3 = J_3'*Qfl22_3*Kld_3 - Qfl12_3*Kld_3 + Pfl11_3-J_3'*Pfl21_3;
Rfls_3 = Rfl22_3 + Kfl_3'*Qfl22_3*Kfl_3 - Pfl22_3'*Kfl_3 - Kfl_3'*Pfl22_3;
Pfls_3 = Kld_3'*Qfl22_3*Kfl_3 - Kld_3'*Pfl22_3 + Rfl12_3 - Pfl21_3'*Kfl_3;
Tfls_3 = Kld_3'*Qfl22_3*Kld_3 - Kld_3'*Pfl21_3 - Pfl21_3'*Kld_3 + Rfl11_3;

Qfls_4 = Qfl11_4 - Qfl12_4*J_4 - J_4'*Qfl21_4 + J_4'*Qfl22_4*J_4;
Ufls_4 = J_4'*Qfl22_4*Kfl_4 - Qfl12_4*Kfl_4 + Pfl12_4-J_4'*Pfl22_4;
Ulds_4 = J_4'*Qfl22_4*Kld_4 - Qfl12_4*Kld_4 + Pfl11_4-J_4'*Pfl21_4;
Rfls_4 = Rfl22_4 + Kfl_4'*Qfl22_4*Kfl_4 - Pfl22_4'*Kfl_4 - Kfl_4'*Pfl22_4;
Pfls_4 = Kld_4'*Qfl22_4*Kfl_4 - Kld_4'*Pfl22_4 + Rfl12_4 - Pfl21_4'*Kfl_4;
Tfls_4 = Kld_4'*Qfl22_4*Kld_4 - Kld_4'*Pfl21_4 - Pfl21_4'*Kld_4 + Rfl11_4;

Qfls_5 = Qfl11_5 - Qfl12_5*J_5 - J_5'*Qfl21_5 + J_5'*Qfl22_5*J_5;
Ufls_5 = J_5'*Qfl22_5*Kfl_5 - Qfl12_5*Kfl_5 + Pfl12_5-J_5'*Pfl22_5;
Ulds_5 = J_5'*Qfl22_5*Kld_5 - Qfl12_5*Kld_5 + Pfl11_5-J_5'*Pfl21_5;
Rfls_5 = Rfl22_5 + Kfl_5'*Qfl22_5*Kfl_5 - Pfl22_5'*Kfl_5 - Kfl_5'*Pfl22_5;
Pfls_5 = Kld_5'*Qfl22_5*Kfl_5 - Kld_5'*Pfl22_5 + Rfl12_5 - Pfl21_5'*Kfl_5;
Tfls_5 = Kld_5'*Qfl22_5*Kld_5 - Kld_5'*Pfl21_5 - Pfl21_5'*Kld_5 + Rfl11_5;

Qfls_6 = Qfl11_6 - Qfl12_6*J_6 - J_6'*Qfl21_6 + J_6'*Qfl22_6*J_6; 
Ufls_6 = J_6'*Qfl22_6*Kfl_6 - Qfl12_6*Kfl_6 + Pfl12_6-J_6'*Pfl22_6;
Ulds_6 = J_6'*Qfl22_6*Kld_6 - Qfl12_6*Kld_6 + Pfl11_6-J_6'*Pfl21_6;
Rfls_6 = Rfl22_6 + Kfl_6'*Qfl22_6*Kfl_6 - Pfl22_6'*Kfl_6 - Kfl_6'*Pfl22_6;
Pfls_6 = Kld_6'*Qfl22_6*Kfl_6 - Kld_6'*Pfl22_6 + Rfl12_6 - Pfl21_6'*Kfl_6;
Tfls_6 = Kld_6'*Qfl22_6*Kld_6 - Kld_6'*Pfl21_6 - Pfl21_6'*Kld_6 + Rfl11_6; 

bfvbf_1 = bet*(PP(1,1)*Bfl_11'*Vflt1_1*Bfl_11 + PP(2,1)*Bfl_12'*Vflt1_2*Bfl_12 + PP(3,1)*Bfl_13'*Vflt1_3*Bfl_13 + PP(4,1)*Bfl_14'*Vflt1_4*Bfl_14 + PP(5,1)*Bfl_15'*Vflt1_5*Bfl_15  + PP(6,1)*Bfl_16'*Vflt1_6*Bfl_16);
bfvbf_2 = bet*(PP(1,2)*Bfl_21'*Vflt1_1*Bfl_21 + PP(2,2)*Bfl_22'*Vflt1_2*Bfl_22 + PP(3,2)*Bfl_23'*Vflt1_3*Bfl_23 + PP(4,2)*Bfl_24'*Vflt1_4*Bfl_24 + PP(5,2)*Bfl_25'*Vflt1_5*Bfl_25  + PP(6,2)*Bfl_26'*Vflt1_6*Bfl_26);
bfvbf_3 = bet*(PP(1,3)*Bfl_31'*Vflt1_1*Bfl_31 + PP(2,3)*Bfl_32'*Vflt1_2*Bfl_32 + PP(3,3)*Bfl_33'*Vflt1_3*Bfl_33 + PP(4,3)*Bfl_34'*Vflt1_4*Bfl_34 + PP(5,3)*Bfl_35'*Vflt1_5*Bfl_35  + PP(6,3)*Bfl_36'*Vflt1_6*Bfl_36);
bfvbf_4 = bet*(PP(1,4)*Bfl_41'*Vflt1_1*Bfl_41 + PP(2,4)*Bfl_42'*Vflt1_2*Bfl_42 + PP(3,4)*Bfl_43'*Vflt1_3*Bfl_43 + PP(4,4)*Bfl_44'*Vflt1_4*Bfl_44 + PP(5,4)*Bfl_45'*Vflt1_5*Bfl_45  + PP(6,4)*Bfl_46'*Vflt1_6*Bfl_46);
bfvbf_5 = bet*(PP(1,5)*Bfl_51'*Vflt1_1*Bfl_51 + PP(2,5)*Bfl_52'*Vflt1_2*Bfl_52 + PP(3,5)*Bfl_53'*Vflt1_3*Bfl_53 + PP(4,5)*Bfl_54'*Vflt1_4*Bfl_54 + PP(5,5)*Bfl_55'*Vflt1_5*Bfl_55  + PP(6,5)*Bfl_56'*Vflt1_6*Bfl_56);
bfvbf_6 = bet*(PP(1,6)*Bfl_61'*Vflt1_1*Bfl_61 + PP(2,6)*Bfl_62'*Vflt1_2*Bfl_62 + PP(3,6)*Bfl_63'*Vflt1_3*Bfl_63 + PP(4,6)*Bfl_64'*Vflt1_4*Bfl_64 + PP(5,6)*Bfl_65'*Vflt1_5*Bfl_65  + PP(6,6)*Bfl_66'*Vflt1_6*Bfl_66);

bfva_1  = bet*(PP(1,1)*Bfl_11'*Vflt1_1*Aj_11 + PP(2,1)*Bfl_12'*Vflt1_2*Aj_12 + PP(3,1)*Bfl_13'*Vflt1_3*Aj_13 + PP(4,1)*Bfl_14'*Vflt1_4*Aj_14 + PP(5,1)*Bfl_15'*Vflt1_5*Aj_15 + PP(6,1)*Bfl_16'*Vflt1_6*Aj_16);
bfva_2  = bet*(PP(1,2)*Bfl_21'*Vflt1_1*Aj_21 + PP(2,2)*Bfl_22'*Vflt1_2*Aj_22 + PP(3,2)*Bfl_23'*Vflt1_3*Aj_23 + PP(4,2)*Bfl_24'*Vflt1_4*Aj_24 + PP(5,2)*Bfl_25'*Vflt1_5*Aj_25 + PP(6,2)*Bfl_26'*Vflt1_6*Aj_26);
bfva_3  = bet*(PP(1,3)*Bfl_31'*Vflt1_1*Aj_31 + PP(2,3)*Bfl_32'*Vflt1_2*Aj_32 + PP(3,3)*Bfl_33'*Vflt1_3*Aj_33 + PP(4,3)*Bfl_34'*Vflt1_4*Aj_34 + PP(5,3)*Bfl_35'*Vflt1_5*Aj_35 + PP(6,3)*Bfl_36'*Vflt1_6*Aj_36);
bfva_4  = bet*(PP(1,4)*Bfl_41'*Vflt1_1*Aj_41 + PP(2,4)*Bfl_42'*Vflt1_2*Aj_42 + PP(3,4)*Bfl_43'*Vflt1_3*Aj_43 + PP(4,4)*Bfl_44'*Vflt1_4*Aj_44 + PP(5,4)*Bfl_45'*Vflt1_5*Aj_45 + PP(6,4)*Bfl_46'*Vflt1_6*Aj_46);
bfva_5  = bet*(PP(1,5)*Bfl_51'*Vflt1_1*Aj_51 + PP(2,5)*Bfl_52'*Vflt1_2*Aj_52 + PP(3,5)*Bfl_53'*Vflt1_3*Aj_53 + PP(4,5)*Bfl_54'*Vflt1_4*Aj_54 + PP(5,5)*Bfl_55'*Vflt1_5*Aj_55 + PP(6,5)*Bfl_56'*Vflt1_6*Aj_56);
bfva_6  = bet*(PP(1,6)*Bfl_61'*Vflt1_1*Aj_61 + PP(2,6)*Bfl_62'*Vflt1_2*Aj_62 + PP(3,6)*Bfl_63'*Vflt1_3*Aj_63 + PP(4,6)*Bfl_64'*Vflt1_4*Aj_64 + PP(5,6)*Bfl_65'*Vflt1_5*Aj_65 + PP(6,6)*Bfl_66'*Vflt1_6*Aj_66);

bfvbl_1 = bet*(PP(1,1)*Bfl_11'*Vflt1_1*Bld_11 + PP(2,1)*Bfl_12'*Vflt1_2*Bld_12 + PP(3,1)*Bfl_13'*Vflt1_3*Bld_13 + PP(4,1)*Bfl_14'*Vflt1_4*Bld_14 + PP(5,1)*Bfl_15'*Vflt1_5*Bld_15 + PP(6,1)*Bfl_16'*Vflt1_6*Bld_16);
bfvbl_2 = bet*(PP(1,2)*Bfl_21'*Vflt1_1*Bld_21 + PP(2,2)*Bfl_22'*Vflt1_2*Bld_22 + PP(3,2)*Bfl_23'*Vflt1_3*Bld_23 + PP(4,2)*Bfl_24'*Vflt1_4*Bld_24 + PP(5,2)*Bfl_25'*Vflt1_5*Bld_25 + PP(6,2)*Bfl_26'*Vflt1_6*Bld_26);
bfvbl_3 = bet*(PP(1,3)*Bfl_31'*Vflt1_1*Bld_31 + PP(2,3)*Bfl_32'*Vflt1_2*Bld_32 + PP(3,3)*Bfl_33'*Vflt1_3*Bld_33 + PP(4,3)*Bfl_34'*Vflt1_4*Bld_34 + PP(5,3)*Bfl_35'*Vflt1_5*Bld_35 + PP(6,3)*Bfl_36'*Vflt1_6*Bld_36);
bfvbl_4 = bet*(PP(1,4)*Bfl_41'*Vflt1_1*Bld_41 + PP(2,4)*Bfl_42'*Vflt1_2*Bld_42 + PP(3,4)*Bfl_43'*Vflt1_3*Bld_43 + PP(4,4)*Bfl_44'*Vflt1_4*Bld_44 + PP(5,4)*Bfl_45'*Vflt1_5*Bld_45 + PP(6,4)*Bfl_46'*Vflt1_6*Bld_46);
bfvbl_5 = bet*(PP(1,5)*Bfl_51'*Vflt1_1*Bld_51 + PP(2,5)*Bfl_52'*Vflt1_2*Bld_52 + PP(3,5)*Bfl_53'*Vflt1_3*Bld_43 + PP(4,5)*Bfl_54'*Vflt1_4*Bld_54 + PP(5,5)*Bfl_55'*Vflt1_5*Bld_55 + PP(6,5)*Bfl_56'*Vflt1_6*Bld_56);
bfvbl_6 = bet*(PP(1,6)*Bfl_61'*Vflt1_1*Bld_61 + PP(2,6)*Bfl_62'*Vflt1_2*Bld_62 + PP(3,6)*Bfl_63'*Vflt1_3*Bld_63 + PP(4,6)*Bfl_64'*Vflt1_4*Bld_64 + PP(5,6)*Bfl_65'*Vflt1_5*Bld_65 + PP(6,6)*Bfl_66'*Vflt1_6*Bld_66);

aabb_1 = bet*PP(1,1)*(Aj_11'-Fld_1'*Bld_11')*Vflt1_1*(Aj_11-Bld_11*Fld_1) + bet*PP(2,1)*(Aj_12'-Fld_1'*Bld_12')*Vflt1_2*(Aj_12-Bld_12*Fld_1)...  
       + bet*PP(3,1)*(Aj_13'-Fld_1'*Bld_13')*Vflt1_3*(Aj_13-Bld_13*Fld_1) + bet*PP(4,1)*(Aj_14'-Fld_1'*Bld_14')*Vflt1_4*(Aj_14-Bld_14*Fld_1)...
       + bet*PP(5,1)*(Aj_15'-Fld_1'*Bld_15')*Vflt1_5*(Aj_15-Bld_15*Fld_1) + bet*PP(6,1)*(Aj_16'-Fld_1'*Bld_16')*Vflt1_6*(Aj_16-Bld_16*Fld_1); 
       
aabb_2 = bet*PP(1,2)*(Aj_21'-Fld_2'*Bld_21')*Vflt1_1*(Aj_21-Bld_21*Fld_2) + bet*PP(2,2)*(Aj_22'-Fld_2'*Bld_22')*Vflt1_2*(Aj_22-Bld_22*Fld_2)...  
       + bet*PP(3,2)*(Aj_23'-Fld_2'*Bld_23')*Vflt1_3*(Aj_23-Bld_23*Fld_2) + bet*PP(4,2)*(Aj_24'-Fld_2'*Bld_24')*Vflt1_4*(Aj_24-Bld_24*Fld_2)...
       + bet*PP(5,2)*(Aj_25'-Fld_2'*Bld_25')*Vflt1_5*(Aj_25-Bld_25*Fld_2) + bet*PP(6,2)*(Aj_26'-Fld_2'*Bld_26')*Vflt1_6*(Aj_26-Bld_26*Fld_2);  
 
aabb_3 = bet*PP(1,3)*(Aj_31'-Fld_3'*Bld_31')*Vflt1_1*(Aj_31-Bld_31*Fld_3) + bet*PP(2,3)*(Aj_32'-Fld_3'*Bld_32')*Vflt1_2*(Aj_32-Bld_32*Fld_3)...  
       + bet*PP(3,3)*(Aj_33'-Fld_3'*Bld_33')*Vflt1_3*(Aj_33-Bld_33*Fld_3) + bet*PP(4,3)*(Aj_34'-Fld_3'*Bld_34')*Vflt1_4*(Aj_34-Bld_34*Fld_3)...
       + bet*PP(5,3)*(Aj_35'-Fld_3'*Bld_35')*Vflt1_5*(Aj_35-Bld_35*Fld_3) + bet*PP(6,3)*(Aj_36'-Fld_3'*Bld_36')*Vflt1_6*(Aj_36-Bld_36*Fld_3); 

aabb_4 = bet*PP(1,4)*(Aj_41'-Fld_4'*Bld_41')*Vflt1_1*(Aj_41-Bld_41*Fld_4) + bet*PP(2,4)*(Aj_42'-Fld_4'*Bld_42')*Vflt1_2*(Aj_42-Bld_42*Fld_4)...  
       + bet*PP(3,4)*(Aj_43'-Fld_4'*Bld_43')*Vflt1_3*(Aj_43-Bld_43*Fld_4) + bet*PP(4,4)*(Aj_44'-Fld_4'*Bld_44')*Vflt1_4*(Aj_44-Bld_44*Fld_4)...
       + bet*PP(5,4)*(Aj_45'-Fld_4'*Bld_45')*Vflt1_5*(Aj_45-Bld_45*Fld_4) + bet*PP(6,4)*(Aj_46'-Fld_4'*Bld_46')*Vflt1_6*(Aj_46-Bld_46*Fld_4);  

aabb_5 = bet*PP(1,5)*(Aj_51'-Fld_5'*Bld_51')*Vflt1_1*(Aj_51-Bld_51*Fld_5) + bet*PP(2,5)*(Aj_52'-Fld_5'*Bld_52')*Vflt1_2*(Aj_52-Bld_52*Fld_5)...  
       + bet*PP(3,5)*(Aj_53'-Fld_5'*Bld_53')*Vflt1_3*(Aj_53-Bld_53*Fld_5) + bet*PP(4,5)*(Aj_54'-Fld_5'*Bld_54')*Vflt1_4*(Aj_54-Bld_54*Fld_5)...
       + bet*PP(5,5)*(Aj_55'-Fld_5'*Bld_55')*Vflt1_5*(Aj_55-Bld_55*Fld_5) + bet*PP(6,5)*(Aj_56'-Fld_5'*Bld_56')*Vflt1_6*(Aj_56-Bld_56*Fld_5);  

aabb_6 = bet*PP(1,6)*(Aj_61'-Fld_6'*Bld_61')*Vflt1_1*(Aj_61-Bld_61*Fld_6) + bet*PP(2,6)*(Aj_62'-Fld_6'*Bld_62')*Vflt1_2*(Aj_62-Bld_62*Fld_6)... 
       + bet*PP(3,6)*(Aj_63'-Fld_6'*Bld_63')*Vflt1_3*(Aj_63-Bld_63*Fld_6) + bet*PP(4,6)*(Aj_64'-Fld_6'*Bld_64')*Vflt1_4*(Aj_64-Bld_64*Fld_6)...
       + bet*PP(5,6)*(Aj_65'-Fld_6'*Bld_65')*Vflt1_5*(Aj_65-Bld_65*Fld_6) + bet*PP(6,6)*(Aj_66'-Fld_6'*Bld_66')*Vflt1_6*(Aj_66-Bld_66*Fld_6);

  
aaa_1 =  bet*PP(1,1)*Bfl_11'*Vflt1_1*(Aj_11-Bld_11*Fld_1) + bet*PP(2,1)*Bfl_12'*Vflt1_2*(Aj_12-Bld_12*Fld_1)...
      +  bet*PP(3,1)*Bfl_13'*Vflt1_3*(Aj_13-Bld_13*Fld_1) + bet*PP(4,1)*Bfl_14'*Vflt1_4*(Aj_14-Bld_14*Fld_1)...
      +  bet*PP(5,1)*Bfl_15'*Vflt1_5*(Aj_15-Bld_15*Fld_1) + bet*PP(6,1)*Bfl_16'*Vflt1_6*(Aj_16-Bld_16*Fld_1);

aaa_2 =  bet*PP(1,2)*Bfl_21'*Vflt1_1*(Aj_21-Bld_21*Fld_2) + bet*PP(2,2)*Bfl_22'*Vflt1_2*(Aj_22-Bld_22*Fld_2)...
      +  bet*PP(3,2)*Bfl_23'*Vflt1_3*(Aj_23-Bld_23*Fld_2) + bet*PP(4,2)*Bfl_24'*Vflt1_4*(Aj_24-Bld_24*Fld_2)...
      +  bet*PP(5,2)*Bfl_25'*Vflt1_5*(Aj_25-Bld_25*Fld_2) + bet*PP(6,2)*Bfl_26'*Vflt1_6*(Aj_26-Bld_26*Fld_2);

aaa_3 =  bet*PP(1,3)*Bfl_31'*Vflt1_1*(Aj_31-Bld_31*Fld_3) + bet*PP(2,3)*Bfl_32'*Vflt1_2*(Aj_32-Bld_32*Fld_3)...
      +  bet*PP(3,3)*Bfl_33'*Vflt1_3*(Aj_33-Bld_33*Fld_3) + bet*PP(4,3)*Bfl_34'*Vflt1_4*(Aj_34-Bld_34*Fld_3)...
      +  bet*PP(5,3)*Bfl_35'*Vflt1_5*(Aj_35-Bld_35*Fld_3) + bet*PP(6,3)*Bfl_36'*Vflt1_6*(Aj_36-Bld_36*Fld_3);

aaa_4 =  bet*PP(1,4)*Bfl_41'*Vflt1_1*(Aj_41-Bld_41*Fld_4) + bet*PP(2,4)*Bfl_42'*Vflt1_2*(Aj_42-Bld_42*Fld_4)...
      +  bet*PP(3,4)*Bfl_43'*Vflt1_3*(Aj_43-Bld_43*Fld_4) + bet*PP(4,4)*Bfl_44'*Vflt1_4*(Aj_44-Bld_44*Fld_4)...
      +  bet*PP(5,4)*Bfl_45'*Vflt1_5*(Aj_45-Bld_45*Fld_4) + bet*PP(6,4)*Bfl_46'*Vflt1_6*(Aj_46-Bld_46*Fld_4);

aaa_5 =  bet*PP(1,5)*Bfl_51'*Vflt1_1*(Aj_51-Bld_51*Fld_5) + bet*PP(2,5)*Bfl_52'*Vflt1_2*(Aj_52-Bld_52*Fld_5)...
      +  bet*PP(3,5)*Bfl_53'*Vflt1_3*(Aj_53-Bld_53*Fld_5) + bet*PP(4,5)*Bfl_54'*Vflt1_4*(Aj_54-Bld_54*Fld_5)...
      +  bet*PP(5,5)*Bfl_55'*Vflt1_5*(Aj_55-Bld_55*Fld_5) + bet*PP(6,5)*Bfl_56'*Vflt1_6*(Aj_56-Bld_56*Fld_5);

aaa_6 =  bet*PP(1,6)*Bfl_61'*Vflt1_1*(Aj_61-Bld_61*Fld_6) + bet*PP(2,6)*Bfl_62'*Vflt1_2*(Aj_62-Bld_62*Fld_6)...
      +  bet*PP(3,6)*Bfl_63'*Vflt1_3*(Aj_63-Bld_63*Fld_6) + bet*PP(4,6)*Bfl_64'*Vflt1_4*(Aj_64-Bld_64*Fld_6)...
      +  bet*PP(5,6)*Bfl_65'*Vflt1_5*(Aj_65-Bld_65*Fld_6) + bet*PP(6,6)*Bfl_66'*Vflt1_6*(Aj_66-Bld_66*Fld_6);
  
% followers' policy rules 
Vfl_1  = Qfls_1 - Fld_1'*Ulds_1' - Ulds_1*Fld_1 + Fld_1'*Tfls_1*Fld_1 + aabb_1...
         -(Ufls_1'-Pfls_1'*Fld_1 + aaa_1)'*inv(Rfls_1 + bfvbf_1)*(Ufls_1'-Pfls_1'*Fld_1 + aaa_1);    
Ffln_1 = inv(Rfls_1 + bfvbf_1) * (Ufls_1' + bfva_1);  %X
Lfln_1 = inv(Rfls_1 + bfvbf_1) * (Pfls_1' + bfvbl_1); %uL
Ct1n_1 = J_1 - Kfl_1*Ffln_1 + Kfl_1*Lfln_1*Fld_1 - Kld_1*Fld_1; %G is obtained given Fld_1 (leader's policy) first introduced

Vfl_2  = Qfls_2 - Fld_2'*Ulds_2' - Ulds_2*Fld_2 + Fld_2'*Tfls_2*Fld_2 + aabb_2... 
         -(Ufls_2'-Pfls_2'*Fld_2 + aaa_2)'*inv(Rfls_2 + bfvbf_2)*(Ufls_2'-Pfls_2'*Fld_2 + aaa_2);
Ffln_2 = inv(Rfls_2 + bfvbf_2) * (Ufls_2' + bfva_2); %X
Lfln_2 = inv(Rfls_2 + bfvbf_2) * (Pfls_2' + bfvbl_2); %uL
Ct1n_2 = J_2 - Kfl_2*Ffln_2 + Kfl_2*Lfln_2*Fld_2 - Kld_2*Fld_2; %G 

Vfl_3  = Qfls_3 - Fld_3'*Ulds_3' - Ulds_3*Fld_3 + Fld_3'*Tfls_3*Fld_3 + aabb_3... 
         -(Ufls_3'-Pfls_3'*Fld_3 + aaa_3)'*inv(Rfls_3 + bfvbf_3)*(Ufls_3'-Pfls_3'*Fld_3 + aaa_3);
Ffln_3 = inv(Rfls_3 + bfvbf_3) * (Ufls_3' + bfva_3); %X
Lfln_3 = inv(Rfls_3 + bfvbf_3) * (Pfls_3' + bfvbl_3); %uL
Ct1n_3 = J_3 - Kfl_3*Ffln_3 + Kfl_3*Lfln_3*Fld_3 - Kld_3*Fld_3;%G 

Vfl_4  = Qfls_4 - Fld_4'*Ulds_4' - Ulds_4*Fld_4 + Fld_4'*Tfls_4*Fld_4 + aabb_4... 
         -(Ufls_4'-Pfls_4'*Fld_4 + aaa_4)'*inv(Rfls_4 + bfvbf_4)*(Ufls_4'-Pfls_4'*Fld_4 + aaa_4);
Ffln_4 = inv(Rfls_4 + bfvbf_4) * (Ufls_4' + bfva_4); %X
Lfln_4 = inv(Rfls_4 + bfvbf_4) * (Pfls_4' + bfvbl_4); %uL
Ct1n_4 = J_4 - Kfl_4*Ffln_4 + Kfl_4*Lfln_4*Fld_4 - Kld_4*Fld_4;%G 

Vfl_5  = Qfls_5 - Fld_5'*Ulds_5' - Ulds_5*Fld_5 + Fld_5'*Tfls_5*Fld_5 + aabb_5... 
         -(Ufls_5'-Pfls_5'*Fld_5 + aaa_5)'*inv(Rfls_5 + bfvbf_5)*(Ufls_5'-Pfls_5'*Fld_5 + aaa_5);
Ffln_5 = inv(Rfls_5 + bfvbf_5) * (Ufls_5' + bfva_5); %X
Lfln_5 = inv(Rfls_5 + bfvbf_5) * (Pfls_5' + bfvbl_5); %uL
Ct1n_5 = J_5 - Kfl_5*Ffln_5 + Kfl_5*Lfln_5*Fld_5 - Kld_5*Fld_5;%G

Vfl_6  = Qfls_6 - Fld_6'*Ulds_6' - Ulds_6*Fld_6 + Fld_6'*Tfls_6*Fld_6 + aabb_6... 
         -(Ufls_6'-Pfls_6'*Fld_6 + aaa_6)'*inv(Rfls_6 + bfvbf_6)*(Ufls_6'-Pfls_6'*Fld_6 + aaa_6);
Ffln_6 = inv(Rfls_6 + bfvbf_6) * (Ufls_6' + bfva_6); %X
Lfln_6 = inv(Rfls_6 + bfvbf_6) * (Pfls_6' + bfvbl_6); %uL
Ct1n_6 = J_6 - Kfl_6*Ffln_6 + Kfl_6*Lfln_6*Fld_6 - Kld_6*Fld_6;%G
  
%change the following parts%
Vflt1_1 = step*Vfl_1 + (1-step)*Vflt1_1;
Vflt1_2 = step*Vfl_2 + (1-step)*Vflt1_2;
Vflt1_3 = step*Vfl_3 + (1-step)*Vflt1_3;
Vflt1_4 = step*Vfl_4 + (1-step)*Vflt1_4;
Vflt1_5 = step*Vfl_5 + (1-step)*Vflt1_5;
Vflt1_6 = step*Vfl_6 + (1-step)*Vflt1_6; 

Ffl_1 = step*Ffln_1 + (1-step)*Ffl_1;
Ffl_2 = step*Ffln_2 + (1-step)*Ffl_2;
Ffl_3 = step*Ffln_3 + (1-step)*Ffl_3;
Ffl_4 = step*Ffln_4 + (1-step)*Ffl_4;
Ffl_5 = step*Ffln_5 + (1-step)*Ffl_5;
Ffl_6 = step*Ffln_6 + (1-step)*Ffl_6;

Lfl_1 = step*Lfln_1 + (1-step)*Lfl_1;
Lfl_2 = step*Lfln_2 + (1-step)*Lfl_2;
Lfl_3 = step*Lfln_3 + (1-step)*Lfl_3;
Lfl_4 = step*Lfln_4 + (1-step)*Lfl_4;
Lfl_5 = step*Lfln_5 + (1-step)*Lfl_5;
Lfl_6 = step*Lfln_6 + (1-step)*Lfl_6;

% this part change %
Ct1_1 = step*Ct1n_1 + (1-step)*Ct1_1; 
Ct1_2 = step*Ct1n_2 + (1-step)*Ct1_2;
Ct1_3 = step*Ct1n_3 + (1-step)*Ct1_3;
Ct1_4 = step*Ct1n_4 + (1-step)*Ct1_4;
Ct1_5 = step*Ct1n_5 + (1-step)*Ct1_5;
Ct1_6 = step*Ct1n_6 + (1-step)*Ct1_6;    
    
%*********************************** leader ************************************%   
d1mat_1 = inv(A22_1 + (PP(1,1)*H_1*Ct1_1*A12_1 + PP(2,1)*H_2*Ct1_2*A12_2 + PP(3,1)*H_3*Ct1_3*A12_3 + PP(4,1)*H_4*Ct1_4*A12_4 + PP(5,1)*H_5*Ct1_5*A12_5 + PP(6,1)*H_6*Ct1_6*A12_6));
d1mat_2 = inv(A22_2 + (PP(1,2)*H_1*Ct1_1*A12_1 + PP(2,2)*H_2*Ct1_2*A12_2 + PP(3,2)*H_3*Ct1_3*A12_3 + PP(4,2)*H_4*Ct1_4*A12_4 + PP(5,2)*H_5*Ct1_5*A12_5 + PP(6,2)*H_6*Ct1_6*A12_6));
d1mat_3 = inv(A22_3 + (PP(1,3)*H_1*Ct1_1*A12_1 + PP(2,3)*H_2*Ct1_2*A12_2 + PP(3,3)*H_3*Ct1_3*A12_3 + PP(4,3)*H_4*Ct1_4*A12_4 + PP(5,3)*H_5*Ct1_5*A12_5 + PP(6,3)*H_6*Ct1_6*A12_6));
d1mat_4 = inv(A22_4 + (PP(1,4)*H_1*Ct1_1*A12_1 + PP(2,4)*H_2*Ct1_2*A12_2 + PP(3,4)*H_3*Ct1_3*A12_3 + PP(4,4)*H_4*Ct1_4*A12_4 + PP(5,4)*H_5*Ct1_5*A12_5 + PP(6,4)*H_6*Ct1_6*A12_6));
d1mat_5 = inv(A22_5 + (PP(1,5)*H_1*Ct1_1*A12_1 + PP(2,5)*H_2*Ct1_2*A12_2 + PP(3,5)*H_3*Ct1_3*A12_3 + PP(4,5)*H_4*Ct1_4*A12_4 + PP(5,5)*H_5*Ct1_5*A12_5 + PP(6,5)*H_6*Ct1_6*A12_6));
d1mat_6 = inv(A22_6 + (PP(1,6)*H_1*Ct1_1*A12_1 + PP(2,6)*H_2*Ct1_2*A12_2 + PP(3,6)*H_3*Ct1_3*A12_3 + PP(4,6)*H_4*Ct1_4*A12_4 + PP(5,6)*H_5*Ct1_5*A12_5 + PP(6,6)*H_6*Ct1_6*A12_6));

J_1 = d1mat_1 *((PP(1,1)*H_1*Ct1_1*A11_1 + PP(2,1)*H_2*Ct1_2*A11_2 + PP(3,1)*H_3*Ct1_3*A11_3 + PP(4,1)*H_4*Ct1_4*A11_4 + PP(5,1)*H_5*Ct1_5*A11_5 + PP(6,1)*H_6*Ct1_6*A11_6) + A21_1);
J_2 = d1mat_2 *((PP(1,2)*H_1*Ct1_1*A11_1 + PP(2,2)*H_2*Ct1_2*A11_2 + PP(3,2)*H_3*Ct1_3*A11_3 + PP(4,2)*H_4*Ct1_4*A11_4 + PP(5,2)*H_5*Ct1_5*A11_5 + PP(6,2)*H_6*Ct1_6*A11_6) + A21_2);
J_3 = d1mat_3 *((PP(1,3)*H_1*Ct1_1*A11_1 + PP(2,3)*H_2*Ct1_2*A11_2 + PP(3,3)*H_3*Ct1_3*A11_3 + PP(4,3)*H_4*Ct1_4*A11_4 + PP(5,3)*H_5*Ct1_5*A11_5 + PP(6,3)*H_6*Ct1_6*A11_6) + A21_3);
J_4 = d1mat_4 *((PP(1,4)*H_1*Ct1_1*A11_1 + PP(2,4)*H_2*Ct1_2*A11_2 + PP(3,4)*H_3*Ct1_3*A11_3 + PP(4,4)*H_4*Ct1_4*A11_4 + PP(5,4)*H_5*Ct1_5*A11_5 + PP(6,4)*H_6*Ct1_6*A11_6) + A21_4);
J_5 = d1mat_5 *((PP(1,5)*H_1*Ct1_1*A11_1 + PP(2,5)*H_2*Ct1_2*A11_2 + PP(3,5)*H_3*Ct1_3*A11_3 + PP(4,5)*H_4*Ct1_4*A11_4 + PP(5,5)*H_5*Ct1_5*A11_5 + PP(6,5)*H_6*Ct1_6*A11_6) + A21_5);
J_6 = d1mat_6 *((PP(1,6)*H_1*Ct1_1*A11_1 + PP(2,6)*H_2*Ct1_2*A11_2 + PP(3,6)*H_3*Ct1_3*A11_3 + PP(4,6)*H_4*Ct1_4*A11_4 + PP(5,6)*H_5*Ct1_5*A11_5 + PP(6,6)*H_6*Ct1_6*A11_6) + A21_6);

Kld_1 = d1mat_1 *((PP(1,1)*H_1*Ct1_1*B11_1 + PP(2,1)*H_2*Ct1_2*B11_2 + PP(3,1)*H_3*Ct1_3*B11_3 + PP(4,1)*H_4*Ct1_4*B11_4 + PP(5,1)*H_5*Ct1_5*B11_5 + PP(6,1)*H_6*Ct1_6*B11_6) + B21_1);
Kld_2 = d1mat_2 *((PP(1,2)*H_1*Ct1_1*B11_1 + PP(2,2)*H_2*Ct1_2*B11_2 + PP(3,2)*H_3*Ct1_3*B11_3 + PP(4,2)*H_4*Ct1_4*B11_4 + PP(5,2)*H_5*Ct1_5*B11_5 + PP(6,2)*H_6*Ct1_6*B11_6) + B21_2);
Kld_3 = d1mat_3 *((PP(1,3)*H_1*Ct1_1*B11_1 + PP(2,3)*H_2*Ct1_2*B11_2 + PP(3,3)*H_3*Ct1_3*B11_3 + PP(4,3)*H_4*Ct1_4*B11_4 + PP(5,3)*H_5*Ct1_5*B11_5 + PP(6,3)*H_6*Ct1_6*B11_6) + B21_3);
Kld_4 = d1mat_4 *((PP(1,4)*H_1*Ct1_1*B11_1 + PP(2,4)*H_2*Ct1_2*B11_2 + PP(3,4)*H_3*Ct1_3*B11_3 + PP(4,4)*H_4*Ct1_4*B11_4 + PP(5,4)*H_5*Ct1_5*B11_5 + PP(6,4)*H_6*Ct1_6*B11_6) + B21_4);
Kld_5 = d1mat_5 *((PP(1,5)*H_1*Ct1_1*B11_1 + PP(2,5)*H_2*Ct1_2*B11_2 + PP(3,5)*H_3*Ct1_3*B11_3 + PP(4,5)*H_4*Ct1_4*B11_4 + PP(5,5)*H_5*Ct1_5*B11_5 + PP(6,5)*H_6*Ct1_6*B11_6) + B21_5);
Kld_6 = d1mat_6 *((PP(1,6)*H_1*Ct1_1*B11_1 + PP(2,6)*H_2*Ct1_2*B11_2 + PP(3,6)*H_3*Ct1_3*B11_3 + PP(4,6)*H_4*Ct1_4*B11_4 + PP(5,6)*H_5*Ct1_5*B11_5 + PP(6,6)*H_6*Ct1_6*B11_6) + B21_6);

Kfl_1 = d1mat_1 *((PP(1,1)*H_1*Ct1_1*B12_1 + PP(2,1)*H_2*Ct1_2*B12_2 + PP(3,1)*H_3*Ct1_3*B12_3 + PP(4,1)*H_4*Ct1_4*B12_4 + PP(5,1)*H_5*Ct1_5*B12_5 + PP(6,1)*H_6*Ct1_6*B12_6) + B22_1); 
Kfl_2 = d1mat_2 *((PP(1,2)*H_1*Ct1_1*B12_1 + PP(2,2)*H_2*Ct1_2*B12_2 + PP(3,2)*H_3*Ct1_3*B12_3 + PP(4,2)*H_4*Ct1_4*B12_4 + PP(5,2)*H_5*Ct1_5*B12_5 + PP(6,2)*H_6*Ct1_6*B12_6) + B22_2); 
Kfl_3 = d1mat_3 *((PP(1,3)*H_1*Ct1_1*B12_1 + PP(2,3)*H_2*Ct1_2*B12_2 + PP(3,3)*H_3*Ct1_3*B12_3 + PP(4,3)*H_4*Ct1_4*B12_4 + PP(5,3)*H_5*Ct1_5*B12_5 + PP(6,3)*H_6*Ct1_6*B12_6) + B22_3); 
Kfl_4 = d1mat_4 *((PP(1,4)*H_1*Ct1_1*B12_1 + PP(2,4)*H_2*Ct1_2*B12_2 + PP(3,4)*H_3*Ct1_3*B12_3 + PP(4,4)*H_4*Ct1_4*B12_4 + PP(5,4)*H_5*Ct1_5*B12_5 + PP(6,4)*H_6*Ct1_6*B12_6) + B22_4); 
Kfl_5 = d1mat_5 *((PP(1,5)*H_1*Ct1_1*B12_1 + PP(2,5)*H_2*Ct1_2*B12_2 + PP(3,5)*H_3*Ct1_3*B12_3 + PP(4,5)*H_4*Ct1_4*B12_4 + PP(5,5)*H_5*Ct1_5*B12_5 + PP(6,5)*H_6*Ct1_6*B12_6) + B22_5); 
Kfl_6 = d1mat_6 *((PP(1,6)*H_1*Ct1_1*B12_1 + PP(2,6)*H_2*Ct1_2*B12_2 + PP(3,6)*H_3*Ct1_3*B12_3 + PP(4,6)*H_4*Ct1_4*B12_4 + PP(5,6)*H_5*Ct1_5*B12_5 + PP(6,6)*H_6*Ct1_6*B12_6) + B22_6); 

Jx_1 = J_1 - Kfl_1 * Ffl_1;
Jx_2 = J_2 - Kfl_2 * Ffl_2;
Jx_3 = J_3 - Kfl_3 * Ffl_3;
Jx_4 = J_4 - Kfl_4 * Ffl_4;
Jx_5 = J_5 - Kfl_5 * Ffl_5;
Jx_6 = J_6 - Kfl_6 * Ffl_6;

Kx_1 = Kld_1 - Kfl_1 * Lfl_1; 
Kx_2 = Kld_2 - Kfl_2 * Lfl_2; 
Kx_3 = Kld_3 - Kfl_3 * Lfl_3; 
Kx_4 = Kld_4 - Kfl_4 * Lfl_4; 
Kx_5 = Kld_5 - Kfl_5 * Lfl_5; 
Kx_6 = Kld_6 - Kfl_6 * Lfl_6; 

Bx_11 = Bld_11 - Bfl_11 * Lfl_1;
Bx_12 = Bld_12 - Bfl_12 * Lfl_1;
Bx_13 = Bld_13 - Bfl_13 * Lfl_1;
Bx_14 = Bld_14 - Bfl_14 * Lfl_1;
Bx_15 = Bld_15 - Bfl_15 * Lfl_1;
Bx_16 = Bld_16 - Bfl_16 * Lfl_1;

Bx_21 = Bld_21 - Bfl_21 * Lfl_2;
Bx_22 = Bld_22 - Bfl_22 * Lfl_2;
Bx_23 = Bld_23 - Bfl_23 * Lfl_2;
Bx_24 = Bld_24 - Bfl_24 * Lfl_2;
Bx_25 = Bld_25 - Bfl_25 * Lfl_2;
Bx_26 = Bld_26 - Bfl_26 * Lfl_2;

Bx_31 = Bld_31 - Bfl_31 * Lfl_3;
Bx_32 = Bld_32 - Bfl_32 * Lfl_3;
Bx_33 = Bld_33 - Bfl_33 * Lfl_3;
Bx_34 = Bld_34 - Bfl_34 * Lfl_3;
Bx_35 = Bld_35 - Bfl_35 * Lfl_3;
Bx_36 = Bld_36 - Bfl_36 * Lfl_3;

Bx_41 = Bld_41 - Bfl_41 * Lfl_4;
Bx_42 = Bld_42 - Bfl_42 * Lfl_4;
Bx_43 = Bld_43 - Bfl_43 * Lfl_4;
Bx_44 = Bld_44 - Bfl_44 * Lfl_4;
Bx_45 = Bld_45 - Bfl_45 * Lfl_4;
Bx_46 = Bld_46 - Bfl_46 * Lfl_4;

Bx_51 = Bld_51 - Bfl_51 * Lfl_5;
Bx_52 = Bld_52 - Bfl_52 * Lfl_5;
Bx_53 = Bld_53 - Bfl_53 * Lfl_5;
Bx_54 = Bld_54 - Bfl_54 * Lfl_5;
Bx_55 = Bld_55 - Bfl_55 * Lfl_5;
Bx_56 = Bld_56 - Bfl_56 * Lfl_5;

Bx_61 = Bld_61 - Bfl_61 * Lfl_6;
Bx_62 = Bld_62 - Bfl_62 * Lfl_6;
Bx_63 = Bld_63 - Bfl_63 * Lfl_6;
Bx_64 = Bld_64 - Bfl_64 * Lfl_6;
Bx_65 = Bld_65 - Bfl_65 * Lfl_6;
Bx_66 = Bld_66 - Bfl_66 * Lfl_6;

Ax_11 = Aj_11 - Bfl_11 * Ffl_1; 
Ax_12 = Aj_12 - Bfl_12 * Ffl_1;
Ax_13 = Aj_13 - Bfl_13 * Ffl_1;
Ax_14 = Aj_14 - Bfl_14 * Ffl_1;
Ax_15 = Aj_15 - Bfl_15 * Ffl_1;
Ax_16 = Aj_16 - Bfl_16 * Ffl_1;

Ax_21 = Aj_21 - Bfl_21 * Ffl_2; 
Ax_22 = Aj_22 - Bfl_22 * Ffl_2;
Ax_23 = Aj_23 - Bfl_23 * Ffl_2;
Ax_24 = Aj_24 - Bfl_24 * Ffl_2;
Ax_25 = Aj_25 - Bfl_25 * Ffl_2;
Ax_26 = Aj_26 - Bfl_26 * Ffl_2;

Ax_31 = Aj_31 - Bfl_31 * Ffl_3; 
Ax_32 = Aj_32 - Bfl_32 * Ffl_3;
Ax_33 = Aj_33 - Bfl_33 * Ffl_3;
Ax_34 = Aj_34 - Bfl_34 * Ffl_3;
Ax_35 = Aj_35 - Bfl_35 * Ffl_3;
Ax_36 = Aj_36 - Bfl_36 * Ffl_3;

Ax_41 = Aj_41 - Bfl_41 * Ffl_4; 
Ax_42 = Aj_42 - Bfl_42 * Ffl_4;
Ax_43 = Aj_43 - Bfl_43 * Ffl_4;
Ax_44 = Aj_44 - Bfl_44 * Ffl_4;
Ax_45 = Aj_45 - Bfl_45 * Ffl_4;
Ax_46 = Aj_46 - Bfl_46 * Ffl_4;

Ax_51 = Aj_51 - Bfl_51 * Ffl_5; 
Ax_52 = Aj_52 - Bfl_52 * Ffl_5;
Ax_53 = Aj_53 - Bfl_53 * Ffl_5;
Ax_54 = Aj_54 - Bfl_54 * Ffl_5;
Ax_55 = Aj_55 - Bfl_55 * Ffl_5;
Ax_56 = Aj_56 - Bfl_56 * Ffl_5;

Ax_61 = Aj_61 - Bfl_61 * Ffl_6; 
Ax_62 = Aj_62 - Bfl_62 * Ffl_6;
Ax_63 = Aj_63 - Bfl_63 * Ffl_6;
Ax_64 = Aj_64 - Bfl_64 * Ffl_6;
Ax_65 = Aj_65 - Bfl_65 * Ffl_6;
Ax_66 = Aj_66 - Bfl_66 * Ffl_6; 
     
Qx_1 =  Qld11_1 -Pld12_1*Ffl_1 -Ffl_1'*(Pld12_1'-Rld22_1*Ffl_1) -(Qld12_1-Ffl_1'*Pld22_1')*Jx_1 - Jx_1'*(Qld21_1-Pld22_1*Ffl_1) + Jx_1'*Qld22_1*Jx_1;
Ux_1 =  Jx_1'*Qld22_1*Kx_1 - (Qld12_1 - Ffl_1'*Pld22_1')*Kx_1 + Pld11_1 - Pld12_1*Lfl_1- Ffl_1'*(Rld21_1 - Rld22_1*Lfl_1) - Jx_1'*Pld21_1 + Jx_1'*Pld22_1*Lfl_1;   
Rx_1 =  Kx_1'*Qld22_1*Kx_1 + Rld11_1  - Rld12_1*Lfl_1 - Lfl_1'*(Rld21_1 - Rld22_1*Lfl_1)- Kx_1'*(Pld21_1 - Pld22_1*Lfl_1) -(Pld21_1'-Lfl_1'*Pld22_1')*Kx_1;

Qx_2 =  Qld11_2 -Pld12_2*Ffl_2 -Ffl_2'*(Pld12_2'-Rld22_2*Ffl_2) -(Qld12_2-Ffl_2'*Pld22_2')*Jx_2 - Jx_2'*(Qld21_2-Pld22_2*Ffl_2) + Jx_2'*Qld22_2*Jx_2;
Ux_2 =  Jx_2'*Qld22_2*Kx_2 - (Qld12_2 - Ffl_2'*Pld22_2')*Kx_2 + Pld11_2 - Pld12_2*Lfl_2- Ffl_2'*(Rld21_2 - Rld22_2*Lfl_2) - Jx_2'*Pld21_2 + Jx_2'*Pld22_2*Lfl_2;    
Rx_2 =  Kx_2'*Qld22_2*Kx_2 + Rld11_2  - Rld12_2*Lfl_2 - Lfl_2'*(Rld21_2 - Rld22_2*Lfl_2)- Kx_2'*(Pld21_2 - Pld22_2*Lfl_2) -(Pld21_2'-Lfl_2'*Pld22_2')*Kx_2;

Qx_3 =  Qld11_3 -Pld12_3*Ffl_3 -Ffl_3'*(Pld12_3'-Rld22_3*Ffl_3) -(Qld12_3-Ffl_3'*Pld22_3')*Jx_3 - Jx_3'*(Qld21_3-Pld22_3*Ffl_3) + Jx_3'*Qld22_3*Jx_3;
Ux_3 =  Jx_3'*Qld22_3*Kx_3 - (Qld12_3 - Ffl_3'*Pld22_3')*Kx_3 + Pld11_3 - Pld12_3*Lfl_3- Ffl_3'*(Rld21_3 - Rld22_3*Lfl_3) - Jx_3'*Pld21_3 + Jx_3'*Pld22_3*Lfl_3;    
Rx_3 =  Kx_3'*Qld22_3*Kx_3 + Rld11_3  - Rld12_3*Lfl_3 - Lfl_3'*(Rld21_3 - Rld22_3*Lfl_3)- Kx_3'*(Pld21_3 - Pld22_3*Lfl_3) -(Pld21_3'-Lfl_3'*Pld22_3')*Kx_3;

Qx_4 =  Qld11_4 -Pld12_4*Ffl_4 -Ffl_4'*(Pld12_4'-Rld22_4*Ffl_4) -(Qld12_4-Ffl_4'*Pld22_4')*Jx_4 - Jx_4'*(Qld21_4-Pld22_4*Ffl_4) + Jx_4'*Qld22_4*Jx_4;
Ux_4 =  Jx_4'*Qld22_4*Kx_4 - (Qld12_4 - Ffl_4'*Pld22_4')*Kx_4 + Pld11_4 - Pld12_4*Lfl_4- Ffl_4'*(Rld21_4 - Rld22_4*Lfl_4) - Jx_4'*Pld21_4 + Jx_4'*Pld22_4*Lfl_4;    
Rx_4 =  Kx_4'*Qld22_4*Kx_4 + Rld11_4  - Rld12_4*Lfl_4 - Lfl_4'*(Rld21_4 - Rld22_4*Lfl_4)- Kx_4'*(Pld21_4 - Pld22_4*Lfl_4) -(Pld21_4'-Lfl_4'*Pld22_4')*Kx_4;

Qx_5 =  Qld11_5 -Pld12_5*Ffl_5 -Ffl_5'*(Pld12_5'-Rld22_5*Ffl_5) -(Qld12_5-Ffl_5'*Pld22_5')*Jx_5 - Jx_5'*(Qld21_5-Pld22_5*Ffl_5) + Jx_5'*Qld22_5*Jx_5;
Ux_5 =  Jx_5'*Qld22_5*Kx_5 - (Qld12_5 - Ffl_5'*Pld22_5')*Kx_5 + Pld11_5 - Pld12_5*Lfl_5- Ffl_5'*(Rld21_5 - Rld22_5*Lfl_5) - Jx_5'*Pld21_5 + Jx_5'*Pld22_5*Lfl_5;    
Rx_5 =  Kx_5'*Qld22_5*Kx_5 + Rld11_5  - Rld12_5*Lfl_5 - Lfl_5'*(Rld21_5 - Rld22_5*Lfl_5)- Kx_5'*(Pld21_5 - Pld22_5*Lfl_5) -(Pld21_5'-Lfl_5'*Pld22_5')*Kx_5;

Qx_6 =  Qld11_6 -Pld12_6*Ffl_6 -Ffl_6'*(Pld12_6'-Rld22_6*Ffl_6) -(Qld12_6-Ffl_6'*Pld22_6')*Jx_6 - Jx_6'*(Qld21_6-Pld22_6*Ffl_6) + Jx_6'*Qld22_6*Jx_6;
Ux_6 =  Jx_6'*Qld22_6*Kx_6 - (Qld12_6 - Ffl_6'*Pld22_6')*Kx_6 + Pld11_6 - Pld12_6*Lfl_6- Ffl_6'*(Rld21_6 - Rld22_6*Lfl_6) - Jx_6'*Pld21_6 + Jx_6'*Pld22_6*Lfl_6;    
Rx_6 =  Kx_6'*Qld22_6*Kx_6 + Rld11_6  - Rld12_6*Lfl_6 - Lfl_6'*(Rld21_6 - Rld22_6*Lfl_6)- Kx_6'*(Pld21_6 - Pld22_6*Lfl_6) -(Pld21_6'-Lfl_6'*Pld22_6')*Kx_6;

BxVBx_1 = bet*(PP(1,1)*Bx_11'*Vldt1_1*Bx_11 + PP(2,1)*Bx_12'*Vldt1_2*Bx_12 + PP(3,1)*Bx_13'*Vldt1_3*Bx_13 + PP(4,1)*Bx_14'*Vldt1_4*Bx_14 + PP(5,1)*Bx_15'*Vldt1_5*Bx_15 + PP(6,1)*Bx_16'*Vldt1_6*Bx_16);
BxVBx_2 = bet*(PP(1,2)*Bx_21'*Vldt1_1*Bx_21 + PP(2,2)*Bx_22'*Vldt1_2*Bx_22 + PP(3,2)*Bx_23'*Vldt1_3*Bx_23 + PP(4,2)*Bx_24'*Vldt1_4*Bx_24 + PP(5,2)*Bx_25'*Vldt1_5*Bx_25 + PP(6,2)*Bx_26'*Vldt1_6*Bx_26);
BxVBx_3 = bet*(PP(1,3)*Bx_31'*Vldt1_1*Bx_31 + PP(2,3)*Bx_32'*Vldt1_2*Bx_32 + PP(3,3)*Bx_33'*Vldt1_3*Bx_33 + PP(4,3)*Bx_34'*Vldt1_4*Bx_34 + PP(5,3)*Bx_35'*Vldt1_5*Bx_35 + PP(6,3)*Bx_36'*Vldt1_6*Bx_36);
BxVBx_4 = bet*(PP(1,4)*Bx_41'*Vldt1_1*Bx_41 + PP(2,4)*Bx_42'*Vldt1_2*Bx_42 + PP(3,4)*Bx_43'*Vldt1_3*Bx_43 + PP(4,4)*Bx_44'*Vldt1_4*Bx_44 + PP(5,4)*Bx_45'*Vldt1_5*Bx_45 + PP(6,4)*Bx_46'*Vldt1_6*Bx_46);
BxVBx_5 = bet*(PP(1,5)*Bx_51'*Vldt1_1*Bx_51 + PP(2,5)*Bx_52'*Vldt1_2*Bx_52 + PP(3,5)*Bx_53'*Vldt1_3*Bx_53 + PP(4,5)*Bx_54'*Vldt1_4*Bx_54 + PP(5,5)*Bx_55'*Vldt1_5*Bx_55 + PP(6,5)*Bx_56'*Vldt1_6*Bx_56);
BxVBx_6 = bet*(PP(1,6)*Bx_61'*Vldt1_1*Bx_61 + PP(2,6)*Bx_62'*Vldt1_2*Bx_62 + PP(3,6)*Bx_63'*Vldt1_3*Bx_63 + PP(4,6)*Bx_64'*Vldt1_4*Bx_64 + PP(5,6)*Bx_65'*Vldt1_5*Bx_65 + PP(6,6)*Bx_66'*Vldt1_6*Bx_66);

BxVAx_1 = bet*(PP(1,1)*Bx_11'*Vldt1_1*Ax_11 + PP(2,1)*Bx_12'*Vldt1_2*Ax_12 + PP(3,1)*Bx_13'*Vldt1_3*Ax_13 + PP(4,1)*Bx_14'*Vldt1_4*Ax_14  + PP(5,1)*Bx_15'*Vldt1_5*Ax_15   + PP(6,1)*Bx_16'*Vldt1_6*Ax_16);
BxVAx_2 = bet*(PP(1,2)*Bx_21'*Vldt1_1*Ax_21 + PP(2,2)*Bx_22'*Vldt1_2*Ax_22 + PP(3,2)*Bx_23'*Vldt1_3*Ax_23 + PP(4,2)*Bx_24'*Vldt1_4*Ax_24  + PP(5,2)*Bx_25'*Vldt1_5*Ax_25   + PP(6,2)*Bx_26'*Vldt1_6*Ax_26);
BxVAx_3 = bet*(PP(1,3)*Bx_31'*Vldt1_1*Ax_31 + PP(2,3)*Bx_32'*Vldt1_2*Ax_32 + PP(3,3)*Bx_33'*Vldt1_3*Ax_33 + PP(4,3)*Bx_34'*Vldt1_4*Ax_34  + PP(5,3)*Bx_35'*Vldt1_5*Ax_35   + PP(6,3)*Bx_36'*Vldt1_6*Ax_36);
BxVAx_4 = bet*(PP(1,4)*Bx_41'*Vldt1_1*Ax_41 + PP(2,4)*Bx_42'*Vldt1_2*Ax_42 + PP(3,4)*Bx_43'*Vldt1_3*Ax_43 + PP(4,4)*Bx_44'*Vldt1_4*Ax_44  + PP(5,4)*Bx_45'*Vldt1_5*Ax_45   + PP(6,4)*Bx_46'*Vldt1_6*Ax_46);
BxVAx_5 = bet*(PP(1,5)*Bx_51'*Vldt1_1*Ax_51 + PP(2,5)*Bx_52'*Vldt1_2*Ax_52 + PP(3,5)*Bx_53'*Vldt1_3*Ax_53 + PP(4,5)*Bx_54'*Vldt1_4*Ax_54  + PP(5,5)*Bx_55'*Vldt1_5*Ax_55   + PP(6,5)*Bx_56'*Vldt1_6*Ax_56);
BxVAx_6 = bet*(PP(1,6)*Bx_61'*Vldt1_1*Ax_61 + PP(2,6)*Bx_62'*Vldt1_2*Ax_62 + PP(3,6)*Bx_63'*Vldt1_3*Ax_63 + PP(4,6)*Bx_64'*Vldt1_4*Ax_64  + PP(6,5)*Bx_65'*Vldt1_5*Ax_65   + PP(6,6)*Bx_66'*Vldt1_6*Ax_66);

AxVAx_1 = bet*(PP(1,1)*Ax_11'*Vldt1_1*Ax_11 + PP(2,1)*Ax_12'*Vldt1_2*Ax_12 + PP(3,1)*Ax_13'*Vldt1_3*Ax_13 + PP(4,1)*Ax_14'*Vldt1_4*Ax_14 + PP(5,1)*Ax_15'*Vldt1_5*Ax_15  + PP(6,1)*Ax_16'*Vldt1_6*Ax_16);
AxVAx_2 = bet*(PP(1,2)*Ax_21'*Vldt1_1*Ax_21 + PP(2,2)*Ax_22'*Vldt1_2*Ax_22 + PP(3,2)*Ax_23'*Vldt1_3*Ax_23 + PP(4,2)*Ax_24'*Vldt1_4*Ax_24 + PP(5,2)*Ax_25'*Vldt1_5*Ax_25  + PP(6,2)*Ax_26'*Vldt1_6*Ax_26);
AxVAx_3 = bet*(PP(1,3)*Ax_31'*Vldt1_1*Ax_31 + PP(2,3)*Ax_32'*Vldt1_2*Ax_32 + PP(3,3)*Ax_33'*Vldt1_3*Ax_33 + PP(4,3)*Ax_34'*Vldt1_4*Ax_34 + PP(5,3)*Ax_35'*Vldt1_5*Ax_35  + PP(6,3)*Ax_36'*Vldt1_6*Ax_36);
AxVAx_4 = bet*(PP(1,4)*Ax_41'*Vldt1_1*Ax_41 + PP(2,4)*Ax_42'*Vldt1_2*Ax_42 + PP(3,4)*Ax_43'*Vldt1_3*Ax_43 + PP(4,4)*Ax_44'*Vldt1_4*Ax_44 + PP(5,4)*Ax_45'*Vldt1_5*Ax_45  + PP(6,4)*Ax_46'*Vldt1_6*Ax_46);
AxVAx_5 = bet*(PP(1,5)*Ax_51'*Vldt1_1*Ax_51 + PP(2,5)*Ax_52'*Vldt1_2*Ax_52 + PP(3,5)*Ax_53'*Vldt1_3*Ax_53 + PP(4,5)*Ax_54'*Vldt1_4*Ax_54 + PP(5,5)*Ax_55'*Vldt1_5*Ax_55  + PP(6,5)*Ax_56'*Vldt1_6*Ax_56);
AxVAx_6 = bet*(PP(1,6)*Ax_61'*Vldt1_1*Ax_61 + PP(2,6)*Ax_62'*Vldt1_2*Ax_62 + PP(3,6)*Ax_63'*Vldt1_3*Ax_63 + PP(4,6)*Ax_64'*Vldt1_4*Ax_64 + PP(5,6)*Ax_65'*Vldt1_5*Ax_65  + PP(6,6)*Ax_66'*Vldt1_6*Ax_66);

Vld_1 = Qx_1 +  AxVAx_1 - (BxVAx_1 + Ux_1')' * inv(Rx_1 + BxVBx_1) * (Ux_1' + BxVAx_1); 
Vld_2 = Qx_2 +  AxVAx_2 - (BxVAx_2 + Ux_2')' * inv(Rx_2 + BxVBx_2) * (Ux_2' + BxVAx_2);
Vld_3 = Qx_3 +  AxVAx_3 - (BxVAx_3 + Ux_3')' * inv(Rx_3 + BxVBx_3) * (Ux_3' + BxVAx_3);
Vld_4 = Qx_4 +  AxVAx_4 - (BxVAx_4 + Ux_4')' * inv(Rx_4 + BxVBx_4) * (Ux_4' + BxVAx_4);
Vld_5 = Qx_5 +  AxVAx_5 - (BxVAx_5 + Ux_5')' * inv(Rx_5 + BxVBx_5) * (Ux_5' + BxVAx_5);
Vld_6 = Qx_6 +  AxVAx_6 - (BxVAx_6 + Ux_6')' * inv(Rx_6 + BxVBx_6) * (Ux_6' + BxVAx_6);

BxVldBx_1 = bet*(PP(1,1)*Bx_11'*Vld_1*Bx_11 + PP(2,1)*Bx_12'*Vld_2*Bx_12 + PP(3,1)*Bx_13'*Vld_3*Bx_13 + PP(4,1)*Bx_14'*Vld_4*Bx_14 + PP(5,1)*Bx_15'*Vld_5*Bx_15 + PP(6,1)*Bx_16'*Vld_6*Bx_16);
BxVldBx_2 = bet*(PP(1,2)*Bx_21'*Vld_1*Bx_21 + PP(2,2)*Bx_22'*Vld_2*Bx_22 + PP(3,2)*Bx_23'*Vld_3*Bx_23 + PP(4,2)*Bx_24'*Vld_4*Bx_24 + PP(5,2)*Bx_25'*Vld_5*Bx_25 + PP(6,2)*Bx_26'*Vld_6*Bx_26);
BxVldBx_3 = bet*(PP(1,3)*Bx_31'*Vld_1*Bx_31 + PP(2,3)*Bx_32'*Vld_2*Bx_32 + PP(3,3)*Bx_33'*Vld_3*Bx_33 + PP(4,3)*Bx_34'*Vld_4*Bx_34 + PP(5,3)*Bx_35'*Vld_5*Bx_35 + PP(6,3)*Bx_36'*Vld_6*Bx_36);
BxVldBx_4 = bet*(PP(1,4)*Bx_41'*Vld_1*Bx_41 + PP(2,4)*Bx_42'*Vld_2*Bx_42 + PP(3,4)*Bx_43'*Vld_3*Bx_43 + PP(4,4)*Bx_44'*Vld_4*Bx_44 + PP(5,4)*Bx_45'*Vld_5*Bx_45 + PP(6,4)*Bx_46'*Vld_6*Bx_46);
BxVldBx_5 = bet*(PP(1,5)*Bx_51'*Vld_1*Bx_51 + PP(2,5)*Bx_52'*Vld_2*Bx_52 + PP(3,5)*Bx_53'*Vld_3*Bx_53 + PP(4,5)*Bx_54'*Vld_4*Bx_54 + PP(5,5)*Bx_55'*Vld_5*Bx_55 + PP(6,5)*Bx_56'*Vld_6*Bx_56);
BxVldBx_6 = bet*(PP(1,6)*Bx_61'*Vld_1*Bx_61 + PP(2,6)*Bx_62'*Vld_2*Bx_62 + PP(3,6)*Bx_63'*Vld_3*Bx_63 + PP(4,6)*Bx_64'*Vld_4*Bx_64 + PP(5,6)*Bx_65'*Vld_5*Bx_65 + PP(6,6)*Bx_66'*Vld_6*Bx_66);

BxVldAx_1 = bet*(PP(1,1)*Bx_11'*Vld_1*Ax_11 + PP(2,1)*Bx_12'*Vld_2*Ax_12 + PP(3,1)*Bx_13'*Vld_3*Ax_13 + PP(4,1)*Bx_14'*Vld_4*Ax_14 + PP(5,1)*Bx_15'*Vld_5*Ax_15 + PP(6,1)*Bx_16'*Vld_6*Ax_16);
BxVldAx_2 = bet*(PP(1,2)*Bx_21'*Vld_1*Ax_21 + PP(2,2)*Bx_22'*Vld_2*Ax_22 + PP(3,2)*Bx_23'*Vld_3*Ax_23 + PP(4,2)*Bx_24'*Vld_4*Ax_24 + PP(5,2)*Bx_25'*Vld_5*Ax_25 + PP(6,2)*Bx_26'*Vld_6*Ax_26);
BxVldAx_3 = bet*(PP(1,3)*Bx_31'*Vld_1*Ax_31 + PP(2,3)*Bx_32'*Vld_2*Ax_32 + PP(3,3)*Bx_33'*Vld_3*Ax_33 + PP(4,3)*Bx_34'*Vld_4*Ax_34 + PP(5,3)*Bx_35'*Vld_5*Ax_35 + PP(6,3)*Bx_36'*Vld_6*Ax_36);
BxVldAx_4 = bet*(PP(1,4)*Bx_41'*Vld_1*Ax_41 + PP(2,4)*Bx_42'*Vld_2*Ax_42 + PP(3,4)*Bx_43'*Vld_3*Ax_43 + PP(4,4)*Bx_44'*Vld_4*Ax_44 + PP(5,4)*Bx_45'*Vld_5*Ax_45 + PP(6,4)*Bx_46'*Vld_6*Ax_46);
BxVldAx_5 = bet*(PP(1,5)*Bx_51'*Vld_1*Ax_51 + PP(2,5)*Bx_52'*Vld_2*Ax_52 + PP(3,5)*Bx_53'*Vld_3*Ax_53 + PP(4,5)*Bx_54'*Vld_4*Ax_54 + PP(5,5)*Bx_55'*Vld_5*Ax_55 + PP(6,5)*Bx_56'*Vld_6*Ax_56);
BxVldAx_6 = bet*(PP(1,6)*Bx_61'*Vld_1*Ax_61 + PP(2,6)*Bx_62'*Vld_2*Ax_62 + PP(3,6)*Bx_63'*Vld_3*Ax_63 + PP(4,6)*Bx_64'*Vld_4*Ax_64 + PP(5,6)*Bx_65'*Vld_5*Ax_65 + PP(6,6)*Bx_66'*Vld_6*Ax_66);

Fldn_1 = inv(Rx_1 + BxVldBx_1) * (Ux_1' + BxVldAx_1);
Fldn_2 = inv(Rx_2 + BxVldBx_2) * (Ux_2' + BxVldAx_2);
Fldn_3 = inv(Rx_3 + BxVldBx_3) * (Ux_3' + BxVldAx_3);
Fldn_4 = inv(Rx_4 + BxVldBx_4) * (Ux_4' + BxVldAx_4);
Fldn_5 = inv(Rx_5 + BxVldBx_5) * (Ux_5' + BxVldAx_5);
Fldn_6 = inv(Rx_6 + BxVldBx_6) * (Ux_6' + BxVldAx_6);

Ct2n_1 = J_1 - Kfl_1*Ffl_1  + Kfl_1*Lfl_1*Fldn_1 - Kld_1*Fldn_1; 
Ct2n_2 = J_2 - Kfl_2*Ffl_2  + Kfl_2*Lfl_2*Fldn_2 - Kld_2*Fldn_2; 
Ct2n_3 = J_3 - Kfl_3*Ffl_3  + Kfl_3*Lfl_3*Fldn_3 - Kld_3*Fldn_3; 
Ct2n_4 = J_4 - Kfl_4*Ffl_4  + Kfl_4*Lfl_4*Fldn_4 - Kld_4*Fldn_4; 
Ct2n_5 = J_5 - Kfl_5*Ffl_5  + Kfl_5*Lfl_5*Fldn_5 - Kld_5*Fldn_5; 
Ct2n_6 = J_6 - Kfl_6*Ffl_6  + Kfl_6*Lfl_6*Fldn_6 - Kld_6*Fldn_6; 

Vadiff1 = norm(Vld_1-Vldt1_1);   %leader objective   
Vadiff2 = norm(Vld_2-Vldt1_2);  
Vadiff3 = norm(Vld_3-Vldt1_3); 
Vadiff4 = norm(Vld_4-Vldt1_4);
Vadiff5 = norm(Vld_5-Vldt1_5);
Vadiff6 = norm(Vld_6-Vldt1_6);

Vadifff1 = norm(Vfl_1-Vflt1_1);  %Leader policy 
Vadifff2 = norm(Vfl_2-Vflt1_2);
Vadifff3 = norm(Vfl_3-Vflt1_3);
Vadifff4 = norm(Vfl_4-Vflt1_4);
Vadifff5 = norm(Vfl_5-Vflt1_5);
Vadifff6 = norm(Vfl_6-Vflt1_6);

Fadiff1 = norm(Fld_1-Fldn_1);   %Leader policy 
Fadiff2 = norm(Fld_2-Fldn_2);
Fadiff3 = norm(Fld_3-Fldn_3);
Fadiff4 = norm(Fld_4-Fldn_4);
Fadiff5 = norm(Fld_5-Fldn_5);
Fadiff6 = norm(Fld_6-Fldn_6);

Cndiff1 = norm(Ct2n_1-Ct1_1);   
Cndiff2 = norm(Ct2n_2-Ct1_2);  
Cndiff3 = norm(Ct2n_3-Ct1_3);  
Cndiff4 = norm(Ct2n_4-Ct1_4);  
Cndiff5 = norm(Ct2n_5-Ct1_5);  
Cndiff6 = norm(Ct2n_6-Ct1_6);  

xxx =(Vadiff1+Vadiff2+Vadiff3+Vadiff4+Vadiff5+Vadiff6+Vadifff1+Vadifff2+Vadifff3+Vadifff4+Vadifff5+Vadifff6...
      +Fadiff1+Fadiff2+Fadiff3+Fadiff4+Fadiff5+Fadiff6+Cndiff1+Cndiff2+Cndiff3+Cndiff4+Cndiff5+Cndiff6)/24;

Fld_1 = step*Fldn_1 + (1-step)*Fld_1;
Fld_2 = step*Fldn_2 + (1-step)*Fld_2;
Fld_3 = step*Fldn_3 + (1-step)*Fld_3;
Fld_4 = step*Fldn_4 + (1-step)*Fld_4;
Fld_5 = step*Fldn_5 + (1-step)*Fld_5;
Fld_6 = step*Fldn_6 + (1-step)*Fld_6;

Ct1_1 = step*Ct2n_1 + (1-step)*Ct1_1;
Ct1_2 = step*Ct2n_2 + (1-step)*Ct1_2;
Ct1_3 = step*Ct2n_3 + (1-step)*Ct1_3;
Ct1_4 = step*Ct2n_4 + (1-step)*Ct1_4;
Ct1_5 = step*Ct2n_5 + (1-step)*Ct1_5;
Ct1_6 = step*Ct2n_6 + (1-step)*Ct1_6;

Vldt1_1 = step*Vld_1 + (1-step)*Vldt1_1;
Vldt1_2 = step*Vld_2 + (1-step)*Vldt1_2;
Vldt1_3 = step*Vld_3 + (1-step)*Vldt1_3;
Vldt1_4 = step*Vld_4 + (1-step)*Vldt1_4;
Vldt1_5 = step*Vld_5 + (1-step)*Vldt1_5;
Vldt1_6 = step*Vld_6 + (1-step)*Vldt1_6;

Vflt1_1 = step*Vfl_1 + (1-step)*Vflt1_1;
Vflt1_2 = step*Vfl_2 + (1-step)*Vflt1_2;
Vflt1_3 = step*Vfl_3 + (1-step)*Vflt1_3;
Vflt1_4 = step*Vfl_4 + (1-step)*Vflt1_4;
Vflt1_5 = step*Vfl_5 + (1-step)*Vflt1_5;
Vflt1_6 = step*Vfl_6 + (1-step)*Vflt1_6;  
    
  iter=iter+1;
end

xxx;
iter;

Cj(:,:,1) = Ct1_1; Cj(:,:,2) = Ct1_2; Cj(:,:,3) = Ct1_3; Cj(:,:,4) = Ct1_4; Cj(:,:,5) = Ct1_5; Cj(:,:,6) = Ct1_6;
Fa(1,:)   = Fld_1; Fa(2,:)   = Fld_2; Fa(3,:)   = Fld_3; Fa(4,:)   = Fld_4; Fa(5,:)   = Fld_5; Fa(6,:)   = Fld_6; 
Fb(1,:)   = Ffl_1; Fb(2,:)   = Ffl_2; Fb(3,:)   = Ffl_3; Fb(4,:)   = Ffl_4; Fb(5,:)   = Ffl_5; Fb(6,:)   = Ffl_6; 
Lf(1,:)   = Lfl_1; Lf(2,:)   = Lfl_2; Lf(3,:)   = Lfl_3; Lf(4,:)   = Lfl_4; Lf(5,:)   = Lfl_5; Lf(6,:)   = Lfl_6;



%%%********************************************************************* 
%%% MjK, vob1, vobs...
%%%*********************************************************************  
for j = 1:ns  % t 
    for i = 1:ns % t+1 
     Mtemp = Bm(1:nX,1:nX,i) - Bm(1:nX,nX+1:n,i)*Cj(:,:,j) - Cm(1:nX,1:nld,i)*Fa(j,:) - Cm(1:nX,(nld+1):k,i)*(Fb(j,:)- Lf(j,:)*Fa(j,:));
     Mjk(:,:,j,i)  =  Mtemp;         
     OMex(:,:,j,i) =  [Mtemp                            zeros(nX,nxx)  zeros(nX,nld)   zeros(nX,nfl);
                     -(Cj(:,:,i))*Mtemp                 zeros(nxx,nxx) zeros(nxx,nld)  zeros(nxx,nfl);
                     -(Fa(i,:))*Mtemp                   zeros(nld,nxx) zeros(nld,nld)  zeros(nld,nfl); 
                      (Lf(i,:)*Fa(i,:)-Fb(i,:))*Mtemp   zeros(nfl,nxx) zeros(nfl,nld)  zeros(nfl,nfl)];                           
     OCex(:,:,i)   =   [CC^2;      -Cj(:,:,i)*CC^2;           -Fa(i,:)*CC^2;      (Lf(i,:)*Fa(i,:)-Fb(i,:))*CC^2];
     OC(:,:,i)     =   [eye(nX);   -Cj(:,:,i);                -Fa(i,:);           (Lf(i,:)*Fa(i,:)-Fb(i,:))];     
    end
    
    OMtemp(:,:,j) = Bm(1:nX,1:nX,j) - Bm(1:nX,nX+1:n,j)*Cj(:,:,j) - Cm(1:nX,1:nld,j)*Fa(j,:) - Cm(1:nX,(nld+1):k,j)*(Fb(j,:)- Lf(j,:)*Fa(j,:));
    ONtemp(:,:,j) =   [-(Cj(:,:,j));             
                       -(Fa(j,:));                  
                       (Lf(j,:)*Fa(j,:)-Fb(j,:))]; 
    Im(:,:,j)     =   [eye(nX)                      zeros(nX,nxx)  zeros(nX,nld)   zeros(nX,nfl);
                       -(Cj(:,:,j)*eye(nX))         zeros(nxx,nxx) zeros(nxx,nld)  zeros(nxx,nfl);
                       -(Fa(j,:)  *eye(nX))         zeros(nld,nxx) zeros(nld,nld)  zeros(nld,nfl); 
                        (Lf(j,:)*Fa(j,:)-Fb(j,:))*eye(nX)    zeros(nfl,nxx) zeros(nfl,nld)  zeros(nfl,nfl)];     
end
 
                
%%----------------------------------------------------------------
%%Unconditional variance...
%%----------------------------------------------------------------
OMexj(:,:,1)= PP(1,1)* OMex(:,:,1,1)+ PP(2,1)* OMex(:,:,1,2)+...
              PP(3,1)* OMex(:,:,1,3)+ PP(4,1)* OMex(:,:,1,4)+...
              PP(5,1)* OMex(:,:,1,5)+ PP(6,1)* OMex(:,:,1,6);  % time t state 1
          
OMexj(:,:,2)= PP(1,2)* OMex(:,:,2,1)+ PP(2,2)* OMex(:,:,2,2)+...
              PP(3,2)* OMex(:,:,2,3)+ PP(4,2)* OMex(:,:,2,4)+...  % time t state 2
              PP(5,2)* OMex(:,:,2,5)+ PP(6,2)* OMex(:,:,2,6);
          
OMexj(:,:,3)= PP(1,3)* OMex(:,:,3,1)+ PP(2,3)* OMex(:,:,3,2)+...
              PP(3,3)* OMex(:,:,3,3)+ PP(4,3)* OMex(:,:,3,4)+... 
              PP(5,3)* OMex(:,:,3,5)+ PP(6,3)* OMex(:,:,3,6); 
                    
OMexj(:,:,4)= PP(1,4)* OMex(:,:,4,1)+ PP(2,4)* OMex(:,:,4,2)+...
              PP(3,4)* OMex(:,:,4,3)+ PP(4,4)* OMex(:,:,4,4)+... 
              PP(5,4)* OMex(:,:,4,5)+ PP(6,4)* OMex(:,:,4,6);    
          
OMexj(:,:,5)= PP(1,5)* OMex(:,:,5,1)+ PP(2,5)* OMex(:,:,5,2)+...
              PP(3,5)* OMex(:,:,5,3)+ PP(4,5)* OMex(:,:,5,4)+... 
              PP(5,5)* OMex(:,:,5,5)+ PP(6,5)* OMex(:,:,5,6);           
          
OMexj(:,:,6)= PP(1,6)* OMex(:,:,6,1)+ PP(2,6)* OMex(:,:,6,2)+...
              PP(3,6)* OMex(:,:,6,3)+ PP(4,6)* OMex(:,:,6,4)+... 
              PP(5,6)* OMex(:,:,6,5)+ PP(6,6)* OMex(:,:,6,6);             
          
OMexmean = ppp(1,1)* OMexj(:,:,1)+ ppp(1,2)* OMexj(:,:,2)+...
           ppp(1,3)* OMexj(:,:,3)+ppp(1,4)* OMexj(:,:,4)+...
           ppp(1,5)* OMexj(:,:,5)+ppp(1,6)* OMexj(:,:,6);
                     
OCexmean = ppp(1,1)* OCex(:,:,1)+ ppp(1,2)* OCex(:,:,2)+...
           ppp(1,3)* OCex(:,:,3)+ppp(1,4)* OCex(:,:,4)+...
           ppp(1,5)* OCex(:,:,5)+ppp(1,6)* OCex(:,:,6); 
       
OCmean = ppp(1,1)* OC(:,:,1)+ ppp(1,2)* OC(:,:,2)+...
         ppp(1,3)* OC(:,:,3)+ppp(1,4)* OC(:,:,4)+...
         ppp(1,5)* OC(:,:,5)+ppp(1,6)* OC(:,:,6);   
                   
OCexmean8 = OCexmean(1:nX,1:nX); 

NT = ppp(1,1)*ONtemp(:,:,1)+ ppp(1,2)*ONtemp(:,:,2)+ppp(1,3)* ONtemp(:,:,3)+...
     ppp(1,4)*ONtemp(:,:,4)+ ppp(1,5)*ONtemp(:,:,5)+ppp(1,6)* ONtemp(:,:,6); 
 


%% ----------------------------------------------------------------
%  IRF 
%%---------------------------------------------------------------- 
% observation indices 
eq_ygr = 1;
eq_pi  = 2;
eq_r   = 3;
eq_tau = 4;
eq_g_y = 5;
eq_b_y = 6;
eq_tr_y = 7;

%create system matrices for state space model
ZZ= zeros(ny,size(OMex,1));
ZZ(eq_ygr,vi_y)        =  1;
ZZ(eq_ygr,vi_ylag)     = -1;
ZZ(eq_ygr,vi_tech)     =  1;
ZZ(eq_pi,vi_pi)        =  4;
ZZ(eq_r,vi_i)          =  4;
ZZ(eq_tau,vi_tau)      =  1;
ZZ(eq_g_y,vi_g)        =  1;
ZZ(eq_b_y,vi_debt)     =  1/4;
ZZ(eq_tr_y,vi_Tr)      =  1;

%steady-state
DDD = zeros(ny,1);           
DDD(eq_ygr,1)  = ga_mma;
DDD(eq_pi,1)   = p_i_ss;
DDD(eq_r,1)    = p_i_ss + rr + 4*ga_mma;
DDD(eq_tau,1)  = tau*100;
DDD(eq_g_y,1)  = g_y*100;
DDD(eq_b_y,1)  = pmb_ya*100;
DDD(eq_tr_y,1) = tr*100;


for j = 1:ns   
    
irfh = 101;
yyirf = zeros(irfh,ny);  
Xt = zeros(size(OMex,1),1); 
Xt(vi_Tr,1) = 0.9;
Xt=Im(:,:,j)*Xt; % the first period 
yyirf(1,:) = (ZZ*Xt)'+ DDD';

for m = 2:irfh
    Xt = OMex(:,:,j,j)*Xt; 
    yyirf(m,:) = (ZZ*Xt)'+ DDD';  
end

X = ['IRF - regime',num2str(j)]; 
disp(X)
round(yyirf,3)

end